package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import java.security.cert.*;
import sun.security.util.*;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import java.security.*;
import java.security.spec.*;
import au.net.aba.crypto.provider.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import java.io.*;

/**
 *  <pre>
 *  2921 EH { RECIPIENT, ToBeEnveloped } ::= E {
 *  2922    RECIPIENT,
 *  2923    ToBeEnveloped
 *  2924 } (CONSTRAINED BY { -- H(ToBeEnveloped) included in the OAEP block -- })
 *
 *  </pre>
 */
public class EH implements DerEncoder {

	private SecretKey key;
	private EnvelopedData envData;
	private static int[] id_desCBC = {1,3,14,3,2,7};
	public static ObjectIdentifier DES_CBC_oid = new ObjectIdentifier(id_desCBC);
	private ContentInfo toBeEnveloped;
	private OAEP OAEPBlock;

	/**
	 *	@param priv - Private Key used for decrypting enveloped data
	 */	
	public EH(PrivateKey priv, DerInputStream din) throws Exception {
		Security.addProvider(new ABAProvider());
		envData = new EnvelopedData(din);
		getSecretKey(priv);
		getToBeEnveloped();
		if(!verify())
			throw new FalseVerifiedException("Invalid EH, hashes are not same");
		else;
			//System.out.println("Verify ok");
	}

	/**
	 *	@param pub - Public Key used for encrypting toBeEnveloped
	 *	@param name - Recipient's name
	 *	@param serialNum - Recipient's certificate serial number
	 *	@param toBeEnveloped - the data wants to be enveloped, the data in ContentInfo form
	 */ 
	public EH(PublicKey pub, X500Name name, BigInt serialNum, ContentInfo toBeEnveloped) throws Exception {
		this.toBeEnveloped=toBeEnveloped;
		EncryptedContentInfo eci = createEncryptedContentInfo();
		RecipientInfo ri = createRecipientInfo(pub, name, serialNum);
		RecipientInfo[] ris = new RecipientInfo[1];
		ris[0] = ri;
		envData = new EnvelopedData(ris,eci);
	}

	/**
	 *	@param r_cert - Recipient's certificate
	 *	@param toBeEnveloped - the data wants to be enveloped, the data in ContentInfo form
	 */
	public EH(X509Certificate r_cert, ContentInfo toBeEnveloped) throws Exception {
		this.toBeEnveloped = toBeEnveloped;
		EncryptedContentInfo eci = createEncryptedContentInfo();
		RecipientInfo ri = createRecipientInfo(r_cert.getPublicKey(), 
								   (X500Name) r_cert.getIssuerDN(), 
								   new BigInt(r_cert.getSerialNumber()));
		RecipientInfo[] ris = new RecipientInfo[1];
		ris[0] = ri;
		envData = new EnvelopedData(ris,eci);	
	}


	private boolean verify() throws Exception {
		byte[] hash1 = OAEPBlock.getHD();

		MessageDigest sha = MessageDigest.getInstance("SHA");
		sha.update(toBeEnveloped.getContent().toByteArray());
		byte[] hash2 = sha.digest();

		return MessageDigest.isEqual(hash1,hash2);
	}	

	private EncryptedContentInfo createEncryptedContentInfo() throws Exception {
		
		//Generate key
		KeyGenerator keyGen = KeyGenerator.getInstance("DES", "ABA");
     		keyGen.init(new SecureRandom());
     		key = keyGen.generateKey();
		HexDumpEncoder hde = new HexDumpEncoder();		
		System.out.println("Secret Key :" + hde.encodeBuffer(key.getEncoded()));

		//Convert toBeEnveloped.content to bytes of DER
		byte[] content = toBeEnveloped.getContent().toByteArray();
		//put the content into ByteArrayInputStream
		ByteArrayInputStream bin = new ByteArrayInputStream(content);
		
		//Initialize the cipher, get the IV
		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding", "ABA");
		cipher.init(Cipher.ENCRYPT_MODE, key);
		byte[] IV = cipher.getIV();
		System.out.println("IV :" + hde.encodeBuffer(IV));
				
		//Create algorithm identifier including the IV parameter
		AlgorithmId algorithm = 
			new AlgorithmId(DES_CBC_oid, new DerValue(DerValue.tag_OctetString, IV));
		
		//Encrypt toBeEnveloped
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		CipherOutputStream cout = new CipherOutputStream(bout,cipher);
		byte[] block = new byte[1024];
		int len;
		while(bin.available() != 0) {
			len = bin.read(block);
			cout.write(block,0,len);	
		}
		cout.close();
		
		//Create EncryptedContentInfo
		return new EncryptedContentInfo(toBeEnveloped.getContentType(),
						        algorithm,
						        bout.toByteArray());			
	}	

	private RecipientInfo createRecipientInfo(PublicKey pub, X500Name name, BigInt serialNum) throws Exception {
						
		MessageDigest sha = MessageDigest.getInstance("SHA");		
		sha.update(toBeEnveloped.getContent().toByteArray());
		
		//put the secret key into OAEP block
		OAEP newOAEP = new OAEP(key.getEncoded(),sha.digest(),null);	

		HexDumpEncoder hde = new HexDumpEncoder();
		//System.out.println(hde.encodeBuffer(key.getEncoded()));
		
		//return
		return new RecipientInfo(name,
						 serialNum,
						 new AlgorithmId(AlgorithmId.RSA_oid),
						 newOAEP.encrypt(pub));
				
	}	

	private void getSecretKey(PrivateKey priv) throws Exception {
		RecipientInfo[] ris = envData.getRecipientInfos();
	
		AlgorithmId algorithm = ris[0].getKeyEncryptionAlgorithm();
		if(!algorithm.equals(AlgorithmId.RSA_oid))
			throw new RuntimeException("Invalid Algorithm for RecipientInfo");

		byte[] encryptedOAEP = ris[0].getEncryptedKey();

		OAEPBlock = OAEP.decrypt(priv,encryptedOAEP);
		byte[] encodedKey = OAEPBlock.getDEK();

		//HexDumpEncoder hde = new HexDumpEncoder();		
		//System.out.println("Decoded Secret Key :" + hde.encodeBuffer(encodedKey));

		SecretKeySpec keyspec = new SecretKeySpec(encodedKey, "DES");
     		SecretKeyFactory factory = SecretKeyFactory.getInstance("DES", "ABA");
     		key = factory.generateSecret(keyspec);
	}

	private void getToBeEnveloped() throws Exception {
		EncryptedContentInfo eci = envData.getEncryptedContentInfo();
		
		AlgorithmId algorithm = eci.getContentEncryptionAlgorithm();
		if(!algorithm.equals(DES_CBC_oid))
			throw new RuntimeException("Invalid algorithm for EncryptedContentInfo");	
		
		byte[] encContent = eci.getEncryptedContent();
		
		byte[] param = algorithm.getEncodedParams();
		DerValue derIv = new DerValue(param);
		byte[]  IV = derIv.getOctetString();
		
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println("IV :" + hde.encodeBuffer(IV));
		IvParameterSpec ivSpec = new IvParameterSpec(IV);
		
		Cipher cipher = Cipher.getInstance("DES/CBC/PKCS5Padding", "ABA");
		cipher.init(Cipher.DECRYPT_MODE,key,ivSpec);

		ByteArrayInputStream bin = new ByteArrayInputStream(encContent);
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		CipherOutputStream cout = new CipherOutputStream(bout,cipher);
		
		byte[] block = new byte[1024];
		int len;
		while(bin.available() != 0) {
			len = bin.read(block);
			cout.write(block,0,len);	
		}
		cout.close();

		//tanda kebingungan
		toBeEnveloped = new ContentInfo(eci.getContentType(), new DerValue(bout.toByteArray()));
	}

	public void derEncode(OutputStream out) throws java.io.IOException {
		envData.derEncode(out);
	}

	public void encode(DerOutputStream dout) throws IOException {
		envData.encode(dout);
	}

	public ContentInfo getContent(){
		return toBeEnveloped;
	}	

	public BigInt getRecipientCertificateSerialNumber(){
		return envData.getRecipientInfos()[0].getCertificateSerialNumber();
	}

	public X500Name getRecipientIssuerName()
  	{
    		return envData.getRecipientInfos()[0].getIssuerName();
  	}

	public String toString() {
		String s = "";
		s  +=  envData.toString();
		s  +=  "toBeEnveloped :\n" + toBeEnveloped;
		return s;
	}

	public static void main (String[] args) {
	   try {
		Security.addProvider(new ABAProvider());
		/*
	      KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA", "ABA");
		keyGen.initialize(1024, new SecureRandom());
     		KeyPair pair = keyGen.generateKeyPair();

	     	PrivateKey priv = pair.getPrivate();
		PublicKey pub = pair.getPublic();

		X500Name name = new X500Name("Iduy", "DIGSEC", "Fasilkom", "Indonesia");	
		BigInt serial = new BigInt(1234567890);
		ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						new DerValue(DerValue.tag_UniversalString, 
						"tekad kami sudah bulat, tahi kami sudah bulat".getBytes()));

		EH encryption = new EH(pub,name,serial,cInfo);

		DerOutputStream dout = new DerOutputStream();
		System.out.println("Before Encode\n" + encryption);

		encryption.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println("\nDer Encode:\n" + hde.encodeBuffer(dout.toByteArray()));
			
		encryption = new EH(priv, new DerInputStream(dout.toByteArray()));
		System.out.println("After Decode\n" + encryption);
		*/

		ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						new DerValue(DerValue.tag_UniversalString, 
						"tekad kami sudah bulat, tahi kami sudah bulat".getBytes()));

		InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
 		CertificateFactory cf = CertificateFactory.getInstance("X.509");
  		X509Certificate cert = (X509Certificate)cf.generateCertificate(inStream);
 		inStream.close();


		EH encryption = new EH(cert,cInfo);

		DerOutputStream dout = new DerOutputStream();
		System.out.println("Before Encode\n" + encryption);

		encryption.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println("\nDer Encode:\n" + hde.encodeBuffer(dout.toByteArray()));
		
		inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
		java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
		ks.load(inStream,"prayudi".toCharArray());
		PrivateKey priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());		
	
		encryption = new EH(priv, new DerInputStream(dout.toByteArray()));
		System.out.println("After Decode\n" + encryption);

		encryption.encode(dout);
		System.out.println("\nDer Encode:\n" + hde.encodeBuffer(dout.toByteArray()));
		
		encryption = new EH(priv, new DerInputStream(dout.toByteArray()));
		System.out.println("After Decode\n" + encryption);

	   }
	   catch(Exception e) {
		e.printStackTrace();
	   }	
	}
}