package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import java.io.*;
import java.util.Arrays;
import sun.security.util.*;
import sun.security.x509.*;
import sun.misc.HexDumpEncoder;
import java.security.*;
import java.security.cert.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;
import cryptix.provider.Cryptix;

/**
 *	<pre>
 *	2840 HMAC { ToBeHashed, Key } ::= Digest
 *	2841   (CONSTRAINED BY { -- HMAC keyed digest of -- ToBeHashed,
 *	2842                                                       -- using -- Key })
 *	
 *	2744 Digest ::= OCTET STRING (SIZE(1..20))
 *	</pre>
 */
public class HMAC implements DerEncoder
{

	private byte[] key;
	private ContentInfo toBeHashed;
	private byte[] digest;

	/**
	 * Constructor
	 *
	 * @param ci something to be hashed, in ContentInfo form
	 * @param k key used for digesting data
	 */
	public HMAC(ContentInfo ci, byte[] k) throws Exception {
		toBeHashed = ci;
		key = k;
		digest = makeDigest(toBeHashed,key);		
	}

	/**
	 * Constructor
	 *
	 * @param din der value of encoded HMAC
	 */
	public HMAC(DerInputStream din) throws Exception {
		digest = din.getOctetString();		
	}

	/**
	 * verify the message digest
	 *
	 * @param ci data to be verified
	 * @param k key for verifying the digest
	 */
	public boolean verify(ContentInfo ci, byte[] k) throws Exception {
		byte[] md = makeDigest(ci,k);
		return Arrays.equals(md,digest);
	}

	/**
	 * Method for get DER Value 
	 *
	 * @param out OutputStream for storing the result
	 */
	public void derEncode(OutputStream out) throws java.io.IOException {
		//DerValue der = new DerValue(DerValue.tag_OctetString, digest);
		DerOutputStream dout = new DerOutputStream();
		//dout.putDerValue(der);
		dout.putOctetString(digest);
		out.write(dout.toByteArray());
	}

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws java.io.IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "Object HMAC\n";
        	if(toBeHashed != null)
        		s = s + "EK :: toBeHashed: \n" + toBeHashed + "\n";
        	if(digest != null) {
			HexDumpEncoder hde = new HexDumpEncoder();
        		s = s + "EK :: Digest: \n" + hde.encodeBuffer(digest) + "\n";
		}

        return s;
	}

	//make message digest using HMAC with SHA-1 Algorithm
	private byte[] makeDigest(ContentInfo ci, byte[] k) throws Exception {
		//convert content from ContentInfo into array of bytes 
		DerValue derValue = ci.getContent();
		byte[] data = derValue.toByteArray();

		//make the message digest		
		MessageDigest hmac = MessageDigest.getInstance("HMAC-SHA-1");
		((Parameterized) hmac).setParameter("key", k);
		byte[] mac = hmac.digest(data);
		
		return mac;
	}

	static void main(String[] args) {
		try {
			Security.addProvider(new Cryptix());

			//make ContentInfo
			String strParam = "Peringatan pemerintah, merokok membahayakan keuangan anda, ";
			//strParam = strParam + "terutama jika rokoknya adalah rokok-rokok luar yang mahal";
			byte[] param = strParam.getBytes();
                  
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,param));		

			byte[] key = {1,2,3,4,5};

			//mulai construct object
			HMAC hmac = new HMAC(cInfo, key);
			System.out.println(hmac.toString());

			DerOutputStream dout = new DerOutputStream();
			hmac.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
		
			System.out.println("********reconstruct*********");
			HMAC hmac2 = new HMAC(new DerInputStream(dout.toByteArray()));
			System.out.println(hmac2.toString());

			dout.reset();
			hmac2.encode(dout);
			HMAC hmac3 = new HMAC(new DerInputStream(dout.toByteArray()));
			System.out.println(hmac3.toString());

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


}