package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import digsec.set.payment.component.*;
import java.io.*;
import java.util.*;
import java.security.*;
import sun.misc.HexDumpEncoder;
import javax.crypto.*;
import sun.security.util.*;
import au.net.aba.crypto.provider.*;


/** 
 *      <pre>
 *      OAEP - Optimal Asymmetric Encryption Padding
 *      Descryption:
 *      OAEP block = E(Recipient Pbkey, R) with RSA
 *      Item                    Descryption                               Length (byte)
 *      ---------------------------------------------------------------
 *      R                       R = I|PDB                                       128     
 *      I                       bit8=0, bit7-1 random non zero  1       
 *      PDB                     PDB = A|B                                       127     
 *      A                       A = H1(E-Salt)^DB                               111     
 *      H1(t)                   H(t|00)|H(t|01)|...H(t|05)              111     
 *                              H : sha-1
 *      E-Salt          16 bytes random salt                    16      
 *      DB                      DB = BT|BC|V|ADB                                111                     
 *      BT                      Block Type 0x03                         1       
 *      BC                      Block Content, indicating what  1       
 *                              data is carried ADB
 *      V                       Verification String, 7 bytes of 0       7
 *      ADB                     ADB=DEK|HD|X                            102
 *      DEK                     DES key                                 8
 *      HD                      Hash of the content encrypted with      20
 *                              the DES                                 
 *      X                       Extra Parameter                         varies
 *      B                       B = E-Salt ^ H2(A)                      16
 *      H2(t)                   return trailing 16 bytes of sha-1       16
 *                              hash of its argument t   
 *      ---------------------------------------------------------------
 *      </pre>
 *      @author DIGSEC/SET - Fasilkom UI        
 */ 

public class OAEP{                              
	
	private byte[] R        =null;
	private byte[] E_Salt   =null;
      private byte[] A        =null;
	private byte[] B        =null;
	private byte   BC       =0;
	private byte[] DEK      =null;  
      private byte[] HD       =null;
	private OAEPEncoder X   =null;
	private HexDumpEncoder hde = new HexDumpEncoder();

	//Constant
	public static byte PAN_DATA     = (byte) 0x01;
	public static byte PAN_DATA0    = (byte) 0x02;
	public static byte PAN_TOKEN    = (byte) 0x03;
	public static byte PAN_ONLY     = (byte) 0x04;
	public static byte ACCT_DATA    = (byte) 0x05;
	
	public static byte HD_INCLUDE    = (byte) 0x80;

	/**
	 *      @param R is non-encrypted OAEP block
	 */             
	public OAEP(byte[] R) throws Exception  {
		this.R=R;
		ByteArrayInputStream bin = new ByteArrayInputStream(R);
		byte I = (byte) bin.read();     
		if((I & (byte)0x80) == (byte)0x80  || I == (byte) 0x00)
			throw new RuntimeException("Bad package of OAEP block (I)"); 
		A = new byte[111];
		bin.read(A,0,111);
		B = new byte[16];
		bin.read(B,0,16);

		getE_Salt();
		getDB();
	}

	/**
	 *    @param DESkey
	 *    @param hashOfData
	 *    @param extraParameter, instance object of 
       *    PANData, PANData0, PANToken, PANOnly, AcctData
	 *      
	 */     
	public OAEP(byte[] DESkey, 
			byte[] hashOfData ,
			OAEPEncoder extra) throws Exception {

		if(extra!=null) {
			if(extra instanceof PANData) {
				BC = (byte) (BC | PAN_DATA);  
				X = extra;
			}
			else if(extra instanceof PANData0) {
				BC = (byte) (BC | PAN_DATA0);
				X = extra;
			}
			else if(extra instanceof PANToken) {
				BC = (byte) (BC | PAN_TOKEN);
				X = extra;
			}
			else if(extra instanceof PANOnly) {
				BC = (byte) (BC | PAN_ONLY);
				X = extra;
			}
			else if(extra instanceof AcctData) {
				BC = (byte) (BC | ACCT_DATA);
				X = extra;
			}
			else
				throw new RuntimeException("Invalid extra parameter of OAEP");
		}
		else
			X = null;
		
		DEK  = DESkey;
		if(hashOfData!=null) {
			HD   = hashOfData;
			BC = (byte) (BC | HD_INCLUDE);
		} 
		else 
			HD = null;                      
	
		createR();
	}     

	private void createR() throws Exception {

		// create the first part, I
		byte I = generateI();
		
		// create the second part PDB
		byte[] PDB = generatePDB();

				
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		bout.write(I);
		bout.write(PDB,0,PDB.length);

		R = bout.toByteArray();         
	}
	
	private byte generateI() {
	
		//generates a fresh random byte I and returns it
		Random random = new Random(317812);

		byte b=(byte)0x00;
	    	while(b==(byte)0x00) {
			b = (byte)(random.nextInt() & 0x7f);
	    	}
		
		//System.out.println("I: " + b);
		return b;
	}

	private byte[] generatePDB() throws Exception
	{
		generateA();
		generateB();
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		bout.write(A);
		bout.write(B);
		byte[] PDB = bout.toByteArray();                

		return PDB;
	}

	private void generateA() throws Exception
	{

		// assign a random value to private member variable E_salt
      
		generateE_Salt();
		byte[] db = generateDB();       
	
		ByteArrayOutputStream baos1 = new ByteArrayOutputStream();
		ByteArrayOutputStream baos2 = new ByteArrayOutputStream();

		// H1 part
		MessageDigest sha1;
		for ( int i = 0 ; i < 6; i ++ ){
			baos2.write(E_Salt);
			baos2.write((byte) i);
			sha1 = MessageDigest.getInstance("SHA");
			sha1.update(baos2.toByteArray());
			baos1.write(sha1.digest());
			baos2.reset();
		}

		// we need only first 111 characters
		ByteArrayInputStream h1str = 
					new ByteArrayInputStream(baos1.toByteArray());
		byte[] h1 = new byte[111];
		h1str.read(h1);

		byte[] A = new byte[111];
		for ( int i = 0; i < A.length; i++){
			A[i] = (byte) ((0xFF&h1[i])^(0xFF&db[i]));
		}
				
		this.A=A;
	}
      
	private void generateE_Salt() {
		E_Salt = new byte[16];
		Random random = new Random();
		random.nextBytes(E_Salt);
	}

	private byte[] generateDB()throws Exception {
		
		byte BT = (byte)0x03;
		
		byte[] V={(byte)0,(byte)0,(byte)0,(byte)0,(byte)0,(byte)0,(byte)0};
		
		// serve to concatenation
		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		// concatenetion of dek|extra to get actual data block
		baos.write(BT);
		baos.write(BC);
		baos.write(V);
		if(DEK!=null)
			baos.write(DEK);
		if(HD!=null)
			baos.write(HD);
		if(X!=null)
			baos.write(X.encodeOAEP());
		byte[] bTbCVdekExtra = baos.toByteArray();

		byte[] DB = new byte[111];
		for(int i = 0; i < DB.length; i++)
			DB[i] = (byte) 0x00;

		// left  justified
		for(int i = 0; i < bTbCVdekExtra.length; i++)
			DB[i] = bTbCVdekExtra[i];

		return DB;
	}

	private void generateB() throws Exception {
			
		MessageDigest sha2 = MessageDigest.getInstance("SHA");
		sha2.update(A);
		ByteArrayInputStream bais = new ByteArrayInputStream(sha2.digest());
		bais.read();
		bais.read();
		bais.read();
		bais.read();
		byte[] h2a = new byte[16];
		bais.read(h2a);

		B = new byte[16];
		for ( int i = 0 ; i<16; i++){
			B[i] =(byte) ((0xFF&h2a[i])^(0xFF&E_Salt[i]));
		} 
		
	}

	//--------------------------------------------------------------------------

	private void getE_Salt() throws Exception{
		
		MessageDigest sha = MessageDigest.getInstance("SHA"); 
		sha.update(A);
		
		ByteArrayInputStream bin = new ByteArrayInputStream(sha.digest());
		bin.read();
		bin.read();
		bin.read();
		bin.read();
		
		byte[] h2a = new byte[16];
		bin.read(h2a,0,16);

		E_Salt = new byte[16];                          
		for ( int i = 0 ; i<16; i++){
			E_Salt[i] =(byte) ((0xFF&h2a[i])^(0xFF&B[i]));
		}
	}

	private void getDB() throws Exception{

		ByteArrayOutputStream baos1 = new ByteArrayOutputStream();
		ByteArrayOutputStream baos2 = new ByteArrayOutputStream();

		// H1 part
		MessageDigest sha1;
		for ( int i = 0 ; i < 6; i ++ ){
			baos2.write(E_Salt);
			baos2.write((byte) i);
			sha1 = MessageDigest.getInstance("SHA");
			sha1.update(baos2.toByteArray());
			baos1.write(sha1.digest());
			baos2.reset();
		}

		// we need only first 111 characters
		ByteArrayInputStream h1str = 
					new ByteArrayInputStream(baos1.toByteArray());
		byte[] h1 = new byte[111];
		h1str.read(h1);

		byte[] DB = new byte[111];
		for ( int i = 0; i < DB.length; i++){
			DB[i] = (byte) ((0xFF&h1[i])^(0xFF&A[i]));
		}               

		ByteArrayInputStream DBStream = new ByteArrayInputStream(DB);
		byte BT = (byte) DBStream.read();
		BC = (byte) DBStream.read();
		byte[] V = new byte[7];
		DBStream.read(V,0,7);
		byte[] V1 = {0x00,0x00,0x00,0x00,0x00,0x00,0x00};               

		if(BT != (byte)0x03 || !Arrays.equals(V,V1))
			throw new RuntimeException("Bad package of OAEP block (BT or V)"); 
		DEK = new byte[8];
		DBStream.read(DEK,0,8);
		
		if((BC & (byte)0x80) == (byte)0x80) { //HD included 
			HD = new byte[20];
			DBStream.read(HD,0,20);
		} 
		
		if((BC & (byte)0x7f) == PAN_DATA) {
			byte[] tempX = new byte[65];
			DBStream.read(tempX,0,65);
			X = (OAEPEncoder) PANData.parseOAEP(tempX);
		}
		else if((BC & (byte)0x7f) == PAN_DATA0) {
			byte[] tempX = new byte[65];
			DBStream.read(tempX,0,65);
			X = (OAEPEncoder) PANData0.parseOAEP(tempX);
		}
		else if((BC & (byte)0x7f) == PAN_TOKEN) {
			byte[] tempX = new byte[45];
			DBStream.read(tempX,0,45);
			X = (OAEPEncoder) PANToken.parseOAEP(tempX);
		}
		else if((BC & (byte)0x7f) == PAN_ONLY) {
			byte[] tempX = new byte[39];
			DBStream.read(tempX,0,39);
			X = (OAEPEncoder) PANOnly.parseOAEP(tempX);
		}
		else if((BC & (byte)0x7f) == ACCT_DATA) {
			byte[] tempX = new byte[94];
			DBStream.read(tempX,0,94);
			X = (OAEPEncoder) AcctData.parseOAEP(tempX);
		}                
	}
	
	/**
	 *      Method for encrypting the OAEP block
	 *      @param pub - PublicKey
	 *      @return array of byte
	 */
      public byte[] encrypt(PublicKey pub) throws Exception {
		Security.addProvider(new ABAProvider());
		Cipher cipher = Cipher.getInstance("RSA","ABA");
		cipher.init(Cipher.ENCRYPT_MODE, pub);
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		CipherOutputStream cout = new CipherOutputStream(bout,cipher);
		cout.write(R);
		cout.close();
		return bout.toByteArray();
	}

	/**
	 *      Method for decrypting the encrypted OAEP block
	 *      @param priv - PrivateKey
	 *      @param cOAEP - encrypted OAEP block
	 *      @return new instance of OAEP
	 */
	public static OAEP decrypt(PrivateKey priv, byte[] cOAEP) throws Exception  {
		Security.addProvider(new ABAProvider());
		Cipher cipher = Cipher.getInstance("RSA","ABA");
		cipher.init(Cipher.DECRYPT_MODE,priv);
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		CipherOutputStream cout = new CipherOutputStream(bout,cipher);
		cout.write(cOAEP);
		cout.close();
		byte[] R = bout.toByteArray();  
		return new OAEP(R);
	}
	
	private byte[] getR() {
		return R;
	}

	/**
	 *      @return DES Key
	 */
	public byte[] getDEK() {
		return DEK;
	}

	/**
	 *      @return Hash of content to be enveloped
	 */
	public byte[] getHD() {
		return HD;
	}

	/**
	 *      @return Object of Extra Parameter
	 */
	public OAEPEncoder getX() {
		return X;
	}       

	public String toString() {

		String s="";
		s += "OAEP Block" + "\n";
		if(DEK!=null)
			s += "DEK:\n" + hde.encodeBuffer(DEK) + "\n";
		if(HD!=null)
			s += "HD :\n" + hde.encodeBuffer(HD) + "\n";
		if(X!=null)
			s += "X  :\n" + X;
		byte[] dodol = new byte[1];
		dodol[0]=BC;
		s += "BC: " + hde.encodeBuffer(dodol) + "\n"; 
		s += "E_Salt:\n" + hde.encodeBuffer(E_Salt) + "\n";
		s += "R :\n" + hde.encodeBuffer(R) + "\n";
		return s;
	}

	public static void main(String[] args) {

	   try {
		KeyGenerator keyGen = KeyGenerator.getInstance("DES", "ABA");
     		keyGen.init(new SecureRandom());
     		SecretKey key = keyGen.generateKey();
		
		KeyPairGenerator keyGen1 = KeyPairGenerator.getInstance("RSA", "ABA");
		keyGen1.initialize(1024, new SecureRandom());
     		KeyPair pair = keyGen1.generateKeyPair();

	     	PrivateKey priv = pair.getPrivate();
		PublicKey pub = pair.getPublic();

		byte[] hash = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};

		PANOnly pan = new PANOnly("1294", hash);

		OAEP block = new OAEP(key.getEncoded(),hash,null);
		System.out.println(block);
		HexDumpEncoder hde = new HexDumpEncoder();
		byte[] encryptedBlock = block.encrypt(pub);
		System.out.println("EncryptedBlock:\n" + hde.encodeBuffer(encryptedBlock));

		System.out.println("*After*");
		//byte[] R = block.getR();
		//block = new OAEP(R);
		block = OAEP.decrypt(priv,encryptedBlock);
		System.out.println(block);
	   }
	   catch(Exception e) {
		e.printStackTrace();
	   }	
	}
}

