package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import java.security.*;
import java.security.cert.*;
import java.security.interfaces.*;
import sun.security.util.*;
import sun.security.x509.X500Name;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;
import sun.misc.HexDumpEncoder;

/**
  * <pre>
  * 2849 S { SIGNER, ToBeSigned } ::= SignedData
  * 2850    (CONSTRAINED BY { SIGNER, -- signs -- ToBeSigned })
  * 2851    (WITH COMPONENTS { ..., contentInfo
  * 2852         (WITH COMPONENTS {
  * 2853                   ..., content PRESENT }) } ^
  * 2854     WITH COMPONENTS { ..., signerInfos (SIZE(1..2)) })

  * 2710 SignedData ::= SEQUENCE {                                      -- PKCS#7
  * 2711    sdVersion         	INTEGER { sdVer2(2) } (sdVer2),
  * 2712    digestAlgorithms  	DigestAlgorithmIdentifiers,
  * 2713    contentInfo       	ContentInfo,
  * 2714    certificates      	[2] IMPLICIT Certificates  OPTIONAL,
  * 2715    crls              	[3] IMPLICIT CRLSequence  OPTIONAL,
  * 2716    signerInfos       	SignerInfos
  * 2717 }
  * </pre>
 */

public class S implements DerEncoder
{
	protected X500Name[] names;
	protected BigInt[] serials;
	protected PrivateKey[] privateKeys;
	protected SignerInfo[] signerInfos;
	protected X509Certificate[] certificates;
	protected ContentInfo contentInfo;
	protected PKCS7 signedData;

	/**
	 * Constructor for the class S
	 * Parameter is two signer
	 *
	 * @param name name of the signer 
	 * @param serial serial number of the signer 
	 * @param key PrivateKey of the signer 
	 * @param cert X509Certificate of the signer, optional 
	 * @param ci something ToBeSigned 
	 */ 
	public S(X500Name[] n, BigInt[] s, PrivateKey[] keys, X509Certificate[] cert, ContentInfo ci ) throws Exception {
		names = n;
		serials = s;
		privateKeys = keys;
		certificates = cert;
		contentInfo = ci;
		signerInfos = new SignerInfo[2];
		signerInfos[0] = makeSignerInfo(n[0], s[0], keys[0], ci);
		signerInfos[1] = makeSignerInfo(n[1], s[1], keys[1], ci);
		signedData = makeSignedData();
	}

	/**
	 * Constructor for the class S
	 * Parameter is two signer
	 * This constructor get the name and serial number from certificate
	 *
	 * @param key PrivateKey of the signer 
	 * @param cert X509Certificate of the signer, optional 
	 * @param ci something ToBeSigned 
	 */ 
	public S(PrivateKey[] keys, X509Certificate[] cert, ContentInfo ci ) throws Exception {
		//get the name of the certicicate
		names = new X500Name[2];
		names[0] = (X500Name)cert[0].getIssuerDN();
		names[1] = (X500Name)cert[1].getIssuerDN();

		//get the serial number
		serials = new BigInt[2];
		serials[0] = new BigInt(cert[0].getSerialNumber());
		serials[1] = new BigInt(cert[1].getSerialNumber());

		privateKeys = keys;
		certificates = cert;
		contentInfo = ci;

		//make signer info
		signerInfos = new SignerInfo[2];
		signerInfos[0] = makeSignerInfo(names[0], serials[0], privateKeys[0], contentInfo);
		signerInfos[1] = makeSignerInfo(names[1], serials[1], privateKeys[1], contentInfo);

		//make SignedData
		signedData = makeSignedData();
	}

	/**
	 * Constructor for the class S
	 * Parameter is from one signer
	 *
	 * @param name name of the signer 
	 * @param serial serial number of the signer 
	 * @param key PrivateKey of the signer 
	 * @param cert X509Certificate of the signer, optional 
	 * @param ci something ToBeSigned 
	 */ 
	public S(X500Name name, BigInt serial, PrivateKey key, X509Certificate cert, ContentInfo ci ) throws Exception {
		names = new X500Name[1];
		names[0] = name;

		serials = new BigInt[1];
		serials[0] = serial;

		privateKeys = new PrivateKey[1];
		privateKeys[0] = key;

		signerInfos = new SignerInfo[1]; 
		signerInfos[0] = makeSignerInfo(name, serial, privateKeys[0], ci); 

		certificates = new X509Certificate[1];
		certificates[0] = cert;

		contentInfo = ci;
		signedData = makeSignedData();
	}

	/**
	 * Constructor for the class S
	 * Parameter is from one signer
	 * This constructor get the name and serial number from certificate
	 *
	 * @param key PrivateKey of the signer 
	 * @param cert X509Certificate of the signer, optional 
	 * @param ci something ToBeSigned 
	 */ 
	public S(PrivateKey key, X509Certificate cert, ContentInfo ci ) throws Exception {
		//get the name and serial number from certificate
		names = new X500Name[1];
		names[0] = (X500Name)cert.getIssuerDN();
		serials = new BigInt[1];
		serials[0] = new BigInt(cert.getSerialNumber());

		privateKeys = new PrivateKey[1];
		privateKeys[0] = key;

		contentInfo = ci;	

		signerInfos = new SignerInfo[1]; 
		signerInfos[0] = makeSignerInfo(names[0], serials[0], privateKeys[0], contentInfo); 

		certificates = new X509Certificate[1];
		certificates[0] = cert;

		signedData = makeSignedData();
	}

	/**
	 * Constructor for the class S
	 * Public keys are from two signer
	 * Construct the object and do the verification
	 *
	 * @param keys PublicKey of the signer 
	 * @param din DER representation of SignedData
	 */ 
	public S(PublicKey[] keys, DerInputStream din) throws Exception {
		signedData = new PKCS7(din);
		contentInfo = signedData.getContentInfo();
		signerInfos = signedData.getSignerInfos();

		if (signerInfos.length > 1) {
			//masing-masing names dan serials dibuatkan tempat untuk dua item
			names = new X500Name[2];
			serials = new BigInt[2];
			//ambil IssuerName dan SerialNumber
			for(int i=0; i<signerInfos.length;i++) {
				names[i] = signerInfos[i].getIssuerName();
				serials[i] = signerInfos[i].getCertificateSerialNumber();	
			}
		}
		else {
			//masing-masing names dan serials dibuatkan tempat untuk satu item
			names = new X500Name[1];
			serials = new BigInt[1];
			//ambil IssuerName dan SerialNumber
			names[0] = signerInfos[0].getIssuerName();
			serials[0] = signerInfos[0].getCertificateSerialNumber();	
		}

		//verify data
		if (!verifyData(contentInfo, signerInfos[0], keys[0]))
			throw new FalseVerifiedException("Verify S");

		if(signerInfos.length > 1 && keys.length > 1) 
			if (!verifyData(contentInfo, signerInfos[1], keys[1]))
				throw new FalseVerifiedException("Verify S");
	}

	/**
	 * Constructor for the class S
	 * Parameter is from one signer
	 * Construct the object and do the verification
	 *
	 * @param key PublicKey of the signer 
	 * @param din DER representation of SignedData
	 */ 
	public S(PublicKey key, DerInputStream din) throws Exception {
		signedData = new PKCS7(din);
		contentInfo = signedData.getContentInfo();
		signerInfos = signedData.getSignerInfos();

		if (signerInfos.length > 1) {
			//masing-masing names dan serials dibuatkan tempat untuk dua item
			names = new X500Name[2];
			serials = new BigInt[2];
			//ambil IssuerName dan SerialNumber
			for(int i=0; i<signerInfos.length;i++) {
				names[i] = signerInfos[i].getIssuerName();
				serials[i] = signerInfos[i].getCertificateSerialNumber();	
			}
		}
		else {
			//masing-masing names dan serials dibuatkan tempat untuk satu item
			names = new X500Name[1];
			serials = new BigInt[1];
			//ambil IssuerName dan SerialNumber
			names[0] = signerInfos[0].getIssuerName();
			serials[0] = signerInfos[0].getCertificateSerialNumber();	
		}

		//verify data
		if (!verifyData(contentInfo, signerInfos[0], key))
			throw new FalseVerifiedException();
	}

	/**
	 * Constructor for the class S
	 *
	 * @param din DER representation of SignedData
	 */ 
	public S(DerInputStream din) throws Exception {
		signedData = new PKCS7(din);
		contentInfo = signedData.getContentInfo();
		signerInfos = signedData.getSignerInfos();

		//ambil certificates dan public key
		certificates = signedData.getCertificates();
		PublicKey[] pubKeys = new PublicKey[certificates.length];
		for(int j=0;j<certificates.length;j++)
			pubKeys[j] = certificates[j].getPublicKey();

		if (signerInfos.length > 1) {
			//masing-masing names dan serials dibuatkan tempat untuk dua item
			names = new X500Name[2];
			serials = new BigInt[2];
			//ambil IssuerName dan SerialNumber
			for(int i=0; i<signerInfos.length;i++) {
				names[i] = signerInfos[i].getIssuerName();
				serials[i] = signerInfos[i].getCertificateSerialNumber();	
			}
		}
		else {
			//masing-masing names dan serials dibuatkan tempat untuk satu item
			names = new X500Name[1];
			serials = new BigInt[1];
			//ambil IssuerName dan SerialNumber
			names[0] = signerInfos[0].getIssuerName();
			serials[0] = signerInfos[0].getCertificateSerialNumber();	
		}

		//verify data
		if (!verifyData(contentInfo, signerInfos[0], pubKeys[0]))
			throw new FalseVerifiedException("Verify S");

		if(signerInfos.length > 1 && pubKeys.length > 1) 
			if (!verifyData(contentInfo, signerInfos[1], pubKeys[1]))
				throw new FalseVerifiedException("Verify S");
	}

	protected byte[] encryptRSA (byte[] plain, Key key) throws Exception {
		int i=0;
		javax.crypto.Cipher c = javax.crypto.Cipher.getInstance("RSA/ECB/PKCS1Padding");

		//inisialisasi untuk enkrip menggunakan private key 
		c.init(c.ENCRYPT_MODE, key);
			
		i = c.getOutputSize(plain.length);
		byte[] cipher=new byte[i];
		i = c.doFinal(plain,0,plain.length,cipher);

		return cipher;
	}

	protected byte[] decryptRSA (byte[] cipher, PublicKey key) throws Exception {
		int i=0;
		javax.crypto.Cipher c = javax.crypto.Cipher.getInstance("RSA/ECB/PKCS1Padding");

		//inisialisasi untuk dekrip menggunakan public key 
		c.init(c.DECRYPT_MODE, key);
			
		i = c.getOutputSize(cipher.length);
		byte[] tempPlain = new byte[i];
		i = c.doFinal(cipher,0,cipher.length,tempPlain);
		byte[] plain = new byte[i];
		System.arraycopy(tempPlain,0,plain,0,i);

		return plain;
	}

	protected boolean verifyData(ContentInfo ci, SignerInfo si, PublicKey key) throws Exception {
		//buat authenticatedAttributes baru dari ContentInfo
		PKCS9Attributes	newAuthAttrib = makePKCS9Attributes(ci);
		byte[] derNewAuthAttrib = newAuthAttrib.getDerEncoding();

		//ambil authenticatedAttributes dari SignerInfo
		PKCS9Attributes authAttrib = si.getAuthenticatedAttributes();
		byte[] derAuthAttrib = authAttrib.getDerEncoding();

		//bandingkan
		for(int i=0; i<derNewAuthAttrib.length; i++)
			if(derNewAuthAttrib[i] != derAuthAttrib[i])
				return false;

		//ambil encryptedDigest dari SignerInfo
		byte[] encryptedDigest = si.getEncryptedDigest();

		//decrypt encryptedDigest untuk mengambil authenticatedAttributes
		byte[] derOldAuthAttrib = decryptRSA(encryptedDigest, key);

		//bandingkan
		for(int i=0; i<derNewAuthAttrib.length; i++)
			if(derOldAuthAttrib [i] != derNewAuthAttrib[i])
				return false;
			
		return true;
	}

	protected PKCS9Attributes makePKCS9Attributes(ContentInfo ci) throws Exception {
		PKCS9Attribute[] PKCS9 = new PKCS9Attribute[2];

		//convert content from ContentInfo into array of bytes 
		DerValue derValue = ci.getContent();
		byte[] data = derValue.toByteArray();

		//make the message digest
		MessageDigest digest = MessageDigest.getInstance("SHA");
		digest.update(data);
		byte[] md = digest.digest();

		PKCS9[0] = new PKCS9Attribute(PKCS9Attribute.CONTENT_TYPE_OID, ci.getContentType());
		PKCS9[1] = new PKCS9Attribute(PKCS9Attribute.MESSAGE_DIGEST_OID, md);

		PKCS9Attributes PKCS9S = new PKCS9Attributes(PKCS9);

		return PKCS9S;
	}

	protected SignerInfo makeSignerInfo(X500Name n, BigInt s, PrivateKey key, ContentInfo ci) throws Exception {
		//set algotithmID: SHA and RSAEncryption
		AlgorithmId[] aid = new AlgorithmId[2];
		aid[0] = new AlgorithmId(AlgorithmId.SHA_oid);
           	aid[1] = new AlgorithmId(AlgorithmId.RSAEncryption_oid);

		//buat authenticated attributes
		PKCS9Attributes authAttrib = makePKCS9Attributes(ci);
		byte[] derAuthAttrib = authAttrib.getDerEncoding();

		//encrypt the authenticated attributes
		byte[] signature = encryptRSA(derAuthAttrib, key);

		//create SignerInfo
		SignerInfo si = new SignerInfo(n,s,aid[0],authAttrib,aid[1],signature,null);

		return si;
	}

	protected PKCS7 makeSignedData() throws Exception {
		AlgorithmId[] aid = new AlgorithmId[2];
		aid[0] = new AlgorithmId(AlgorithmId.SHA_oid);
		aid[1] = new AlgorithmId(AlgorithmId.SHA_oid);
            //ContentInfo contentInfo = new ContentInfo(new ObjectIdentifier("1.2.3.4"),
		//					  new DerValue("Dodol Garuk"));                 
		signedData = new PKCS7(aid,contentInfo,certificates,signerInfos);
		
		return signedData;
	}

	/**
	 * Method for get DER Value of the underlying SignedData (PKCS7)
	 *
	 * @param OutputStream for storing the result
	 */
	public void derEncode(OutputStream out) throws java.io.IOException {
		signedData.encodeSignedData(out);
	}

	/**
	 * Method for get DER Value of the underlying SignedData (PKCS7)
	 *
	 * @param DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws java.io.IOException {
		signedData.encodeSignedData(dout);
	}

	/**
	 * get SignedData
	 *
	 * @return PKCS7 object
	 */
	public PKCS7 getSignedData() throws Exception {
		return signedData;
	}

	/**
	 * get ContentInfo
	 *
	 * @return ContentInfo object
	 */
	public ContentInfo getContentInfo() throws Exception {
		return contentInfo;
	}

	/**
	 * get Certificates
	 *
	 * @return array of X509Certificate object
	 */
	public X509Certificate[] getCertificates() throws Exception {
		return certificates;
	}

	/**
	 * get the issuer's name of certificate
	 *
	 * @return the issuer name 
	 */
	public X500Name[] getIssuerNames() throws Exception {
		return names;
	}

	/**
	 * get Serial Number of the certificate
	 *
	 * @return the serial number
	 */
	public BigInt[] getSerialNumber() throws Exception {
		return serials;
	}

	/**
	 * get the String representation of this object
	 *
	 */
	public String toString() {
		String s = "Object S\n";
/*
	     	if(names != null)
        	{
			s = s + "S :: Signer names: " + names + "\n";
            	for(int i = 0; i < names.length; i++)
                	s = s + "\t" + i + ".   " + names[i] + "\n";
        	}
        	if(serials != null)
        	{
            	s = s + "S :: Serial numbers: \n";
            	for(int j = 0; j < serials.length; j++)
                		s = s + "\t" + j + ".   " + serials[j] + "\n";
        	}
        	if(certificates != null)
        	{
            	s = s + "S :: Certificates: \n";
            	for(int k = 0; k < certificates.length; k++)
                		s = s + "\t" + k + ".   " + "\n" + certificates[k] + "\n";
        	}
		if(contentInfo != null)
        		s = s + "S :: ToBeSigned: \n" + contentInfo + "\n";
*/
		if(signedData != null)
		  	s = s + "S :: SignedData: \n" + signedData;

        return s;
    }

	static void main(String[] args) {
		int KEYSIZE=512;
		PublicKey myPublicKey;

		try {
			Security.addProvider(new ABAProvider());
			
			//make contentInfo
			String strParam = "Peringatan pemerintah, merokok membahayakan keuangan anda, ";
			strParam = strParam + "terutama jika rokoknya adalah rokok-rokok luar yang mahal";
			byte[] param = strParam.getBytes();
                  
			ContentInfo cInfo = new ContentInfo(ContentInfo.SIGNED_DATA_OID,
							            new DerValue(DerValue.tag_OctetString,param));
				
			//make name and serial number
			X500Name[] names = new X500Name[2];
			BigInt[] serials = new BigInt[2];
			X500Name name1 = new X500Name("Arif", "Digisec", "Fasilkom", "Indonesia");
			names[0] = name1;
			BigInt serial1 = new BigInt(1234);
			serials[0] = serial1;
			X500Name name2 = new X500Name("Iduy", "Secasec", "Mabality Mandiri", "Jawa Merdeka");
			names[1] = name2;
			BigInt serial2 = new BigInt(5678);
			serials[1] = serial2;

			//make private keys
			PrivateKey[] privateKeys = new PrivateKey[2];
			PublicKey[] publicKeys = new PublicKey[2];
			byte[] seedBytes = {2,5,3,4,7};
			SecureRandom myRandom = new SecureRandom(seedBytes);
			KeyPairGenerator rsaKeyPairGen = KeyPairGenerator.getInstance("RSA", "ABA");
			rsaKeyPairGen.initialize(KEYSIZE,myRandom);
			KeyPair myKeyPair = rsaKeyPairGen.generateKeyPair();
			privateKeys[0] = myKeyPair.getPrivate();
			publicKeys[0] = myKeyPair.getPublic();

			//make second private key
			byte[] seedBytes2 = {4,6,2,9};
			myRandom.setSeed(seedBytes2);
			rsaKeyPairGen.initialize(KEYSIZE,myRandom);
			myKeyPair = rsaKeyPairGen.generateKeyPair();
			privateKeys[1] = myKeyPair.getPrivate();
			publicKeys[1] = myKeyPair.getPublic();

			DerOutputStream dout = new DerOutputStream();
			if (args.length > 1 && args[0].equalsIgnoreCase("-cert")) {
				//get first certificate
				InputStream inStream = new FileInputStream(args[1]);
				CertificateFactory cf = CertificateFactory.getInstance("X.509");
				X509Certificate[] cert = new X509Certificate[2];
				cert[0] = (X509Certificate)cf.generateCertificate(inStream);
				inStream.close();
				//get second certificate
				InputStream inStream2 = new FileInputStream(args[2]);
				cert[1] = (X509Certificate)cf.generateCertificate(inStream2);
				inStream.close();

				S s = new S(privateKeys, cert, cInfo);
				System.out.print(s.toString());
				s.encode(dout);
			}
			else {
				//make S object
				S s = new S(names, serials, privateKeys, null, cInfo);
				System.out.print(s.toString());
				s.encode(dout);
		
		}

			//coba liat authenticated attribute
/*			PKCS7 sd = s.getSignedData();
			SignerInfo[] si = sd.getSignerInfos();
			PKCS9Attributes authAttr = si[0].getAuthenticatedAttributes();
			if(authAttr == null)
				System.out.println("No authenticated attributes");
			else
				System.out.println(authAttr.toString());
*/

			//reconstruct the S object
			System.out.println("*******reconstruct*******");
			S s2 = new S(publicKeys, new DerInputStream(dout.toByteArray()));
			System.out.print(s2.toString());
			
			dout.reset();
			s2.encode(dout);
			S s3 = new S(publicKeys, new DerInputStream(dout.toByteArray()));
			System.out.print(s3.toString());

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

}


