package digsec.set.core.encapsulation;
import digsec.set.core.crypto.*;
import digsec.set.core.pkcs.*;
import java.io.*;
import java.util.*;
import sun.security.util.*;
import java.security.cert.*;
import sun.security.x509.*;
import java.security.*;
import au.net.aba.crypto.provider.*;
import javax.crypto.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.*;

/**
 * 	2885 EncX { SIGNER, RECIPIENT, T, Parameter } ::= E {
 *	2886    RECIPIENT,
 *	2887    SEQUENCE {
 *	2888        t  T,
 *	2889        s  SO { SIGNER, SEQUENCE { t  T, p  Parameter } }
 *	2890     }
 *	2891 } (CONSTRAINED BY { Parameter -- data, which shall contain a fresh --
 *	2892                     -- nonce 'n', is included in the OAEP block.  -- } )
 */

public class EncX implements DerEncoder {

	//fields
	//Sender
	private X509Certificate[] s_certificates;
	
	//Recipient
	private X509Certificate r_certificate;
	
	//Content to be encapsulated
	private ContentInfo T;

	//Parameter
	private OAEPEncoder parameter;

	//E
	private E envelope;
	private SO sigOnly;


	//Constructor
	/**
	 *	@param s_certs - sender's certificates
	 *	@param r_cert  - recipient's  certificate
	 *	@param T	   - to be encapsulated
	 *	@param param   - extra parameter implements OAEPEncoder
 	 */
	public EncX(X509Certificate[] s_certs, 
			PrivateKey[] s_privs,
		      X509Certificate r_cert, 
		      ContentInfo T, OAEPEncoder param) throws Exception {
		s_certificates = s_certs;
		r_certificate = r_cert;
		parameter = param;
		this.T = T;	
		makeSO(s_privs);
		makeE();
	}

	public EncX(X509Certificate s_cert,
			PrivateKey s_priv,
 		      X509Certificate r_cert, 
		      ContentInfo T, OAEPEncoder param) throws Exception {
		s_certificates = new X509Certificate[1];
		s_certificates[0] = s_cert;
		r_certificate = r_cert;
		parameter = param;
		this.T = T;
		PrivateKey[] s_privs = new PrivateKey[1];
		s_privs[0] = s_priv;
		makeSO(s_privs);
		makeE();
	}

	/**
	 *	r_priv - recipient's private key
	 *	din	 - DerInputStream which contains the DER value of EncX
	 */
	public EncX(PrivateKey r_priv, DerInputStream din) throws Exception {
		envelope = new E(r_priv, din);
		parameter = envelope.getParameter();
		getTSigOnly();
		s_certificates = sigOnly.getCertificates();
	}

	//private method - utility method
	
	private void makeSO(PrivateKey[] s_privs) throws Exception {
		//make SEQ of T and Parameter
		DerOutputStream dout = new DerOutputStream();
		DerEncoder[] de = new DerEncoder[2];
		de[0]=(DerEncoder)T;
		de[1]=(DerEncoder)parameter;
		dout.putOrderedSetOf((byte)48,de);

		//make SO(SIGNER,SEQ{T,P})
		ContentInfo ci = new ContentInfo(T.getContentType(),
							   new DerValue(dout.toByteArray()));
		if(s_certificates.length==1 && s_privs.length==1)
			sigOnly = new SO(s_privs[0],s_certificates[0],ci);
		else if(s_certificates.length==2 && s_privs.length==2)
			sigOnly = new SO(s_privs,s_certificates,ci);
		else
			throw new IllegalArgumentException("EncX");			
	}

	private void makeE() throws Exception {
  		//make SEQ{T,SO(SIGNER,SEQ{T,P})}
		DerOutputStream dout = new DerOutputStream();
		DerEncoder[] de = new DerEncoder[2];
		de[0]=(DerEncoder)T;
		de[1]=(DerEncoder)sigOnly;
		dout.putOrderedSetOf((byte)48,de); 

		//make E(RECIPIENT,SEQ{T,SO(SIGNER,SEQ{T,P})});
		ContentInfo ci = new ContentInfo(T.getContentType(),
							   new DerValue(dout.toByteArray()));
		envelope = new E(r_certificate,ci,parameter);			 		
	}

	public void getTSigOnly() throws Exception {
		//given SEQ{T,SO(SIGNER,SEQ{T,P})}
		ContentInfo ci = envelope.getContent();
		DerValue derContent = ci.getContent();
		DerInputStream din = new DerInputStream(derContent.toByteArray());
		DerValue[] adv = din.getSequence(2);
		T = new ContentInfo(new DerInputStream(adv[0].toByteArray()));
		
		//Verifying the sigOnly
		//make SEQ of T and Parameter
		DerOutputStream dout = new DerOutputStream();
		DerEncoder[] de = new DerEncoder[2];
		de[0]=(DerEncoder)T;
		de[1]=(DerEncoder)parameter;
		dout.putOrderedSetOf((byte)48,de);

		//ContentInfo of SEQ{T,P}
		ContentInfo ci1 = new ContentInfo(T.getContentType(),
							   new DerValue(dout.toByteArray()));

		//Create sigOnly, automatically throws FalseVerifiedException if
		//the content given is not match with the sigOnly
		sigOnly = new SO(new DerInputStream(adv[1].toByteArray()),ci1);
	}
	

	//public method

	public void derEncode(OutputStream out) throws IOException {
		envelope.derEncode(out);
	}

	public void encode(DerOutputStream dout) throws Exception {
		envelope.encode(dout);
	}
	
	public ContentInfo getContentInfo() {
		return T;
	}

	public X509Certificate[] getSenderCertificates() {
		return s_certificates;
	}

	public X509Certificate getSenderCertificate() {
		return s_certificates[0];
	}

	public OAEPEncoder getParameter() {
		return parameter;
	}
	
	public String toString() {
		String s = "EncX\n";
		s += envelope;
		s += T;
		s += parameter;
		return s;
	}

	public static void main(String[] args) {
		try {
			Security.addProvider(new ABAProvider());

			ContentInfo toBeEncapsulated = new ContentInfo(new ObjectIdentifier("1.2.3.4"),
										     new DerValue("Dodol Garut Kamu"));   		
			byte[] nonce = {1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0};
			PANOnly panOnly = new PANOnly("Priharsanta",nonce); 
			
			//make sender's certificate
			X509Certificate[] s_certs = new X509Certificate[2];

			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();

			s_certs[0] = s_cert;
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
			cf = CertificateFactory.getInstance("X.509");
  			s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			s_certs[1] = s_cert;

			//make recipient's certificate
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
 			cf = CertificateFactory.getInstance("X.509");
  			X509Certificate r_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get sender's PrivateKey
			PrivateKey[] s_privs = new PrivateKey[2];
			
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.keystore");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"priharsanta".toCharArray());
			PrivateKey s_priv = (PrivateKey) ks.getKey("priharsanta", "priharsanta".toCharArray());			
			s_privs[0] = s_priv;		
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			s_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			
			s_privs[1] = s_priv;


			EncX encX = new EncX(s_certs,s_privs,r_cert,toBeEncapsulated,panOnly);
			

			System.out.println(encX);	
			
			DerOutputStream dout = new DerOutputStream();
			encX.encode(dout);
			byte[] DerEncX = dout.toByteArray();
			//DerEnc[167] = (byte) 0x00;	

			HexDumpEncoder hde = new HexDumpEncoder();
			//System.out.println(hde.encodeBuffer(dout.toByteArray()));

			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			PrivateKey r_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			


			encX = new EncX(r_priv, new DerInputStream(DerEncX));
			dout.reset();
			encX.encode(dout);

			encX = new EncX(r_priv, new DerInputStream(dout.toByteArray()));			
		
			System.out.println(encX);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

}
