package digsec.set.core.pkcs;
import java.io.*;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	2774 EncryptedContentInfo ::= SEQUENCE {
 *	2775    contentType       ContentType,
 *	2776    contentEncryptionAlgorithm
 *	2777                      AlgorithmIdentifier {{ContentEncryptionAlgorithms}},
 *	2778    encryptedContent  [0] IMPLICIT EncryptedContent  OPTIONAL
 *	2779 }
 *
 *	2781 EncryptedContent ::= OCTET STRING
 *	</pre>
 */

public class EncryptedContentInfo
   implements DerEncoder
{

 /**
  * Construct this object from a DER encoded input stream.
  *
  * @param	dis the DER encoded input stream.
  * @param	flag flag for contentinfo.
  * @exception	ParsingException if there's an error while parsing the input.
  */
   public EncryptedContentInfo(DerInputStream dis)
      throws IOException, ParsingException
   {
   	DerValue[] adv = dis.getSequence(3);

	contentType = new ObjectIdentifier(new DerInputStream(adv[0].toByteArray()));	   
	contentEncryptionAlgorithm = AlgorithmId.parse(adv[1]);
	if((new DerInputStream(adv[2].toByteArray())).peekByte() == -128) {
		adv[2].resetTag((byte)0x04);	
		encryptedContent = adv[2].getOctetString();
	}
	else
		encryptedContent = null;
   }

  /**
   * Construct this object from a bunch of other objects.
   *
   * @param	conType the contentType.
   * @param	algid the contentEncryptionAlgorithm.
   * @param	encContent the encryptedContent.
   */
   public EncryptedContentInfo(ObjectIdentifier conType, AlgorithmId algid)
   {
      this(conType, algid, null);
   }

  /**
   * Construct this object from a bunch of other objects.
   *
   * @param	conType the contentType.
   * @param	algid the contentEncryptionAlgorithm.
   * @param	encContent the encryptedContent.
   */
   public EncryptedContentInfo(ObjectIdentifier conType, AlgorithmId algid,
                               byte encContent[])
   {
      contentType = null;
      contentEncryptionAlgorithm = null;
      encryptedContent = null;
      contentType = conType;
      contentEncryptionAlgorithm = algid;
      if (encContent != null)
         encryptedContent = encContent;
   }

//-------------------------------------------------------------
// Implementation for the DerEncoded interface

  /**
   * Encode this object to an outputstream in a DER encoded format.
   *
   * @param	out the output stream.
   */
   public void derEncode(OutputStream out)
      throws IOException
   {
      DerOutputStream dout = new DerOutputStream();
      dout.putOID(contentType);
      contentEncryptionAlgorithm.encode(dout);
	if(encryptedContent!=null)
      	dout.putDerValue(new DerValue((byte)-128,encryptedContent));
	else
		dout.putNull();
      DerOutputStream dos = new DerOutputStream();
      dos.write((byte)48, dout);
      out.write(dos.toByteArray());
   }
//-------------------------------------------------------------

   public void encode(DerOutputStream out)
      throws IOException
   {
      derEncode(out);
   }

   public ObjectIdentifier getContentType()
   {
	return contentType; 
   }

   public AlgorithmId getContentEncryptionAlgorithm()
   {
      return contentEncryptionAlgorithm;
   }

   public byte[] getEncryptedContent()
   {
      return encryptedContent;
   }

   public String toString()
   {
      String s = "";
      s = s + "Encrypted Content Info Sequence\n\tContent type: " + contentType + "\n";
      s = s + "\tContent Encryption Algorithm: " + contentEncryptionAlgorithm + "\n";
      HexDumpEncoder hde = new HexDumpEncoder();
	if(encryptedContent!=null)
      s = s + "\tEncrypted Content: \n" + hde.encodeBuffer(encryptedContent) + "\n";
      return s;
   }

   private ObjectIdentifier contentType;
   private AlgorithmId contentEncryptionAlgorithm;
   private byte encryptedContent[];

   public static void main (String[] args) {

    try{
	byte[] encContent = {0,1,2,3,4,5,6,7,8,9};
	int[] oid = {1,3,14,3,2,7};
	ObjectIdentifier id_desCBC = new ObjectIdentifier(oid);	
	EncryptedContentInfo enc = 
		new EncryptedContentInfo(ContentInfo.ENCRYPTED_DATA_OID, 
						 new AlgorithmId(id_desCBC),
						 null);
//	System.out.println(enc.toString());
	DerOutputStream dout = new DerOutputStream();
	enc.encode(dout);
	HexDumpEncoder hde = new HexDumpEncoder();
      byte[] DER = dout.toByteArray();
	System.out.println(hde.encodeBuffer(DER));

	enc = new EncryptedContentInfo(new DerInputStream(DER));
	System.out.println(enc.toString());
     }
     catch(Exception e) {
    		e.printStackTrace();
     }
   }	
}
