package digsec.set.core.pkcs;

import java.io.*;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.cert.*;
import java.util.Vector;
import sun.security.util.*;
import sun.security.x509.*;

// Referenced classes of package sun.security.pkcs:
//            ContentInfo, ParsingException, SignerInfo

/** 
 *  	2710 SignedData ::= SEQUENCE {                                      -- PKCS#7
 *	2711    sdVersion         INTEGER { sdVer2(2) } (sdVer2),
 *	2712    digestAlgorithms  DigestAlgorithmIdentifiers,
 *	2713    contentInfo       ContentInfo,
 *	2714    certificates      [0] IMPLICIT Certificates  OPTIONAL,
 *	2715    crls              [1] IMPLICIT CRLSequence  OPTIONAL,
 *	2716    signerInfos       SignerInfos
 *	2717 }
 *
 *    @author Sun
 *    @see ContentInfo
 *    @see ParsingException
 *    @see SignerInfo	 
 */	
public class PKCS7
{

    /**
     *  Constructor of PKCS7
     *  @param object of InputStream which contains DER values of SignedData
     *  The DER values will be automatically parsed into this object
     */		
    public PKCS7(InputStream inputstream)
        throws ParsingException, IOException
    {
        version = null;
        digestAlgorithmIds = null;
        contentInfo = null;
        certificates = null;
        crls = null;
        signerInfos = null;
        oldStyle = false;
        DataInputStream datainputstream = new DataInputStream(inputstream);
        byte abyte0[] = new byte[datainputstream.available()];
        datainputstream.readFully(abyte0);
        parse(new DerInputStream(abyte0));
    }

    /**
     *  Constructor of PKCS7
     *  @param object of DerInputStream which contains DER values of SignedData
     *  The DER values will be automatically parsed into this object
     */	
    public PKCS7(DerInputStream derinputstream)
        throws ParsingException
    {
        version = null;
        digestAlgorithmIds = null;
        contentInfo = null;
        certificates = null;
        crls = null;
        signerInfos = null;
        oldStyle = false;
        parse(derinputstream);
    }

    /**
     *  Constructor of PKCS7
     *  @param array of byte which contains DER values of SignedData
     *  The DER values will be automatically parsed into this object
     */
    public PKCS7(byte abyte0[])
        throws ParsingException
    {
        version = null;
        digestAlgorithmIds = null;
        contentInfo = null;
        certificates = null;
        crls = null;
        signerInfos = null;
        oldStyle = false;
        DerInputStream derinputstream = new DerInputStream(abyte0);
        parse(derinputstream);
    }

    /**
     *  Constructor of PKCS7
     *  The inputs are SignedData sub-objects	
     *  @param array of AlgorithmId  
     *  @param ContentInfo
     *  @param array of X509Certificate	
     *  @param array of SignerInfo
     */	
    public PKCS7(AlgorithmId aalgorithmid[], ContentInfo contentinfo, X509Certificate ax509certificate[], SignerInfo asignerinfo[])
    {
        version = null;
        digestAlgorithmIds = null;
        contentInfo = null;
        certificates = null;
        crls = null;
        signerInfos = null;
        oldStyle = false;
        version = new BigInt(1);
        digestAlgorithmIds = aalgorithmid;
        contentInfo = contentinfo;
        certificates = ax509certificate;
        signerInfos = asignerinfo;
    }

    /**
     *  Method for encoding this object into DER representation
     *  @param OutputStream	
     */ 
    public void encodeSignedData(OutputStream outputstream)
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        encodeSignedData(deroutputstream);
        outputstream.write(deroutputstream.toByteArray());
    }

    /**
     *  Method for encoding this object into DER representation
     *  @param DerOutputStream	
     */	
    public void encodeSignedData(DerOutputStream deroutputstream)
        throws IOException
    {
        DerOutputStream deroutputstream1 = new DerOutputStream();
        deroutputstream1.putInteger(version);
        deroutputstream1.putOrderedSetOf((byte)49, digestAlgorithmIds);
        contentInfo.encode(deroutputstream1);
        if(certificates != null && certificates.length != 0)
        {
            X509CertImpl ax509certimpl[] = new X509CertImpl[certificates.length];
            for(int i = 0; i < certificates.length; i++)
                if(certificates[i] instanceof X509CertImpl)
                    ax509certimpl[i] = (X509CertImpl)certificates[i];
                else
                    try
                    {
                        byte abyte0[] = certificates[i].getEncoded();
                        ax509certimpl[i] = new X509CertImpl(abyte0);
                    }
                    catch(CertificateException certificateexception)
                    {
                        throw new IOException(certificateexception.getMessage());
                    }

            deroutputstream1.putOrderedSetOf((byte)-96, ax509certimpl);
        }
        deroutputstream1.putOrderedSetOf((byte)49, signerInfos);
        DerValue dervalue = new DerValue((byte)48, deroutputstream1.toByteArray());
        ContentInfo contentinfo = new ContentInfo(ContentInfo.SIGNED_DATA_OID, dervalue);
        contentinfo.encode(deroutputstream);
    }

    /** 
     *  Method for achieving set of X509CRL
     *  @return array of X509CRL
     */ 
    public X509CRL[] getCRLs()
    {
        return crls;
    }

    /** 
     *  Method for achieving a X509Certificate
     *  @param bigint  : Certificate Serial Number
     *  @param x500name: Issuer Distinguish Name 
     *  @return	     : X509Certificate
     */
    public X509Certificate getCertificate(BigInt bigint, X500Name x500name)
    {
        if(certificates != null)
        {
            for(int i = 0; i < certificates.length; i++)
            {
                X509Certificate x509certificate = certificates[i];
                X500Name x500name1 = (X500Name)x509certificate.getIssuerDN();
                java.math.BigInteger biginteger = x509certificate.getSerialNumber();
                BigInt bigint1 = new BigInt(biginteger);
                if(bigint.equals(bigint1) && x500name.equals(x500name1))
                    return x509certificate;
            }

        }
        return null;
    }

    /**
     * Method for achieving set of X509Certificate
     */	
    public X509Certificate[] getCertificates()
    {
        return (X509Certificate[])certificates.clone();
    }

    /**
     * Method for achieving ContentInfo
     */
    public ContentInfo getContentInfo()
    {
        return contentInfo;
    }

    /** 
     *  Method for achieving DigestAlgorithmIds
     *  @return array of AlgorithmId
     */ 
    public AlgorithmId[] getDigestAlgorithmIds()
    {
        return digestAlgorithmIds;
    }

    /**
     *  Method for achieving SignerInfos
     *  @return array of SignerInfo
     */ 	
    public SignerInfo[] getSignerInfos()
    {
        return signerInfos;
    }

    /**
     *  Method for achieving this SignedData-version
     */
    public BigInt getVersion()
    {
        return version;
    }

    public boolean isOldStyle()
    {
        return oldStyle;
    }

    private void parse(DerInputStream derinputstream)
        throws ParsingException
    {
        try
        {
            derinputstream.mark(derinputstream.available());
            parse(derinputstream, false);
        }
        catch(IOException _ex)
        {
            try
            {
                derinputstream.reset();
                parse(derinputstream, true);
                oldStyle = true;
            }
            catch(IOException ioexception)
            {
                ParsingException parsingexception = new ParsingException("IOException: " + ioexception.getMessage());
                parsingexception.fillInStackTrace();
                throw parsingexception;
            }
        }
    }

    private void parse(DerInputStream derinputstream, boolean flag)
        throws IOException
    {
        contentInfo = new ContentInfo(derinputstream, flag);
        contentType = contentInfo.contentType;
        DerValue dervalue = contentInfo.getContent();
        if(contentType.equals(ContentInfo.SIGNED_DATA_OID))
            parseSignedData(dervalue);
        else
        if(contentType.equals(ContentInfo.OLD_SIGNED_DATA_OID))
            parseOldSignedData(dervalue);
        else
        if(contentType.equals(ContentInfo.NETSCAPE_CERT_SEQUENCE_OID))
            parseNetscapeCertChain(dervalue);
        else
            throw new ParsingException("content type " + contentType + " not supported.");
    }

    private void parseNetscapeCertChain(DerValue dervalue)
        throws ParsingException, IOException
    {
        DerInputStream derinputstream = new DerInputStream(dervalue.toByteArray());
        DerValue adervalue[] = derinputstream.getSequence(2);
        certificates = new X509Certificate[adervalue.length];
        CertificateFactory certificatefactory = null;
        try
        {
            certificatefactory = CertificateFactory.getInstance("X.509");
        }
        catch(CertificateException _ex) {}
        for(int i = 0; i < adervalue.length; i++)
        {
            ByteArrayInputStream bytearrayinputstream = null;
            try
            {
                if(certificatefactory == null)
                {
                    certificates[i] = new X509CertImpl(adervalue[i]);
                }
                else
                {
                    byte abyte0[] = adervalue[i].toByteArray();
                    bytearrayinputstream = new ByteArrayInputStream(abyte0);
                    certificates[i] = (X509Certificate)certificatefactory.generateCertificate(bytearrayinputstream);
                    bytearrayinputstream.close();
                    bytearrayinputstream = null;
                }
            }
            catch(CertificateException certificateexception)
            {
                throw new ParsingException("CertificateException: " + certificateexception.getMessage());
            }
            catch(IOException ioexception)
            {
                throw new ParsingException("CertificateException: " + ioexception.getMessage());
            }
            finally
            {
                if(bytearrayinputstream != null)
                    bytearrayinputstream.close();
            }
        }

    }

    private void parseOldSignedData(DerValue dervalue)
        throws ParsingException, IOException
    {
        DerInputStream derinputstream = dervalue.toDerInputStream();
        version = derinputstream.getInteger();
        DerValue adervalue[] = derinputstream.getSet(1);
        int i = adervalue.length;
        digestAlgorithmIds = new AlgorithmId[i];
        try
        {
            for(int j = 0; j < i; j++)
            {
                DerValue dervalue1 = adervalue[j];
                digestAlgorithmIds[j] = AlgorithmId.parse(dervalue1);
            }

        }
        catch(IOException _ex)
        {
            throw new ParsingException("Error parsing digest AlgorithmId IDs");
        }
        contentInfo = new ContentInfo(derinputstream, true);
        CertificateFactory certificatefactory = null;
        try
        {
            certificatefactory = CertificateFactory.getInstance("X.509");
        }
        catch(CertificateException _ex) {}
        DerValue adervalue1[] = derinputstream.getSet(2);
        i = adervalue1.length;
        certificates = new X509Certificate[i];
        for(int k = 0; k < i; k++)
        {
            ByteArrayInputStream bytearrayinputstream = null;
            try
            {
                if(certificatefactory == null)
                {
                    certificates[k] = new X509CertImpl(adervalue1[k]);
                }
                else
                {
                    byte abyte0[] = adervalue1[k].toByteArray();
                    bytearrayinputstream = new ByteArrayInputStream(abyte0);
                    certificates[k] = (X509Certificate)certificatefactory.generateCertificate(bytearrayinputstream);
                    bytearrayinputstream.close();
                    bytearrayinputstream = null;
                }
            }
            catch(CertificateException certificateexception)
            {
                throw new ParsingException("CertificateException: " + certificateexception.getMessage());
            }
            catch(IOException ioexception)
            {
                throw new ParsingException("CertificateException: " + ioexception.getMessage());
            }
            finally
            {
                if(bytearrayinputstream != null)
                    bytearrayinputstream.close();
            }
        }

        derinputstream.getSet(0);
        DerValue adervalue2[] = derinputstream.getSet(1);
        i = adervalue2.length;
        signerInfos = new SignerInfo[i];
        for(int l = 0; l < i; l++)
        {
            DerInputStream derinputstream1 = adervalue2[l].toDerInputStream();
            signerInfos[l] = new SignerInfo(derinputstream1, true);
        }

    }

    private void parseSignedData(DerValue dervalue)
        throws ParsingException, IOException
    {
        DerInputStream derinputstream = dervalue.toDerInputStream();
        version = derinputstream.getInteger();
        DerValue adervalue[] = derinputstream.getSet(1);
        int i = adervalue.length;
        digestAlgorithmIds = new AlgorithmId[i];
        try
        {
            for(int j = 0; j < i; j++)
            {
                DerValue dervalue1 = adervalue[j];
                digestAlgorithmIds[j] = AlgorithmId.parse(dervalue1);
            }

        }
        catch(IOException ioexception)
        {
            ParsingException parsingexception = new ParsingException("Error parsing digest AlgorithmId IDs: " + ioexception.getMessage());
            parsingexception.fillInStackTrace();
            throw parsingexception;
        }
        contentInfo = new ContentInfo(derinputstream);
        if((byte)derinputstream.peekByte() == -96)
        {
            DerValue adervalue1[] = derinputstream.getSet(2, true);
            i = adervalue1.length;
            certificates = new X509Certificate[i];
            CertificateFactory certificatefactory = null;
            try
            {
                certificatefactory = CertificateFactory.getInstance("X.509");
            }
            catch(CertificateException _ex) {}
            for(int i1 = 0; i1 < i; i1++)
            {
                ByteArrayInputStream bytearrayinputstream = null;
                try
                {
                    if(certificatefactory == null)
                    {
                        certificates[i1] = new X509CertImpl(adervalue1[i1]);
                    }
                    else
                    {
                        byte abyte0[] = adervalue1[i1].toByteArray();
                        bytearrayinputstream = new ByteArrayInputStream(abyte0);
                        certificates[i1] = (X509Certificate)certificatefactory.generateCertificate(bytearrayinputstream);
                        bytearrayinputstream.close();
                        bytearrayinputstream = null;
                    }
                }
                catch(CertificateException certificateexception)
                {
                    throw new ParsingException("CertificateException: " + certificateexception.getMessage());
                }
                catch(IOException ioexception1)
                {
                    throw new ParsingException("CertificateException: " + ioexception1.getMessage());
                }
                finally
                {
                    if(bytearrayinputstream != null)
                        bytearrayinputstream.close();
                }
            }

        }
        if((byte)derinputstream.peekByte() == -95)
        {
            DerValue adervalue2[] = derinputstream.getSet(1, true);
            i = adervalue2.length;
            crls = new X509CRL[i];
            for(int k = 0; k < i; k++)
                try
                {
                    X509CRLImpl x509crlimpl = new X509CRLImpl(adervalue2[k]);
                    crls[k] = x509crlimpl;
                }
                catch(CRLException crlexception)
                {
                    ParsingException parsingexception1 = new ParsingException("CRLException: " + crlexception.getMessage());
                    parsingexception1.fillInStackTrace();
                    throw parsingexception1;
                }

        }
        DerValue adervalue3[] = derinputstream.getSet(1);
        i = adervalue3.length;
        signerInfos = new SignerInfo[i];
        for(int l = 0; l < i; l++)
        {
            DerInputStream derinputstream1 = adervalue3[l].toDerInputStream();
            signerInfos[l] = new SignerInfo(derinputstream1);
        }

    }

    /**
     *  Method for converting this SignedData object into String to be used on printing
     */
    public String toString()
    {
        String s = "";
        s = s + contentInfo + "\n";
        if(version != null)
            s = s + "PKCS7 :: version: " + version + "\n";
        if(digestAlgorithmIds != null)
        {
            s = s + "PKCS7 :: digest AlgorithmIds: \n";
            for(int i = 0; i < digestAlgorithmIds.length; i++)
                s = s + "\t" + digestAlgorithmIds[i] + "\n";

        }
        if(certificates != null)
        {
            s = s + "PKCS7 :: certificates: \n";
            for(int j = 0; j < certificates.length; j++)
                s = s + "\t" + j + ".   " + certificates[j] + "\n";

        }
        if(crls != null)
        {
            s = s + "PKCS7 :: crls: \n";
            for(int k = 0; k < crls.length; k++)
                s = s + "\t" + k + ".   " + crls[k] + "\n";

        }
        if(signerInfos != null)
        {
            s = s + "PKCS7 :: signer infos: \n";
            for(int l = 0; l < signerInfos.length; l++)
                s = s + "\t" + l + ".  " + signerInfos[l] + "\n";

        }
        return s;
    }

    public SignerInfo[] verify()
        throws NoSuchAlgorithmException, SignatureException
    {
        return verify(null);
    }

    public SignerInfo verify(SignerInfo signerinfo, byte abyte0[])
        throws NoSuchAlgorithmException, SignatureException
    {
        return signerinfo.verify(this, abyte0);
    }

    public SignerInfo[] verify(byte abyte0[])
        throws NoSuchAlgorithmException, SignatureException
    {
        Vector vector = new Vector();
        for(int i = 0; i < signerInfos.length; i++)
        {
            SignerInfo signerinfo = verify(signerInfos[i], abyte0);
            if(signerinfo != null)
                vector.addElement(signerinfo);
        }

        if(vector.size() != 0)
        {
            SignerInfo asignerinfo[] = new SignerInfo[vector.size()];
            vector.copyInto(asignerinfo);
            return asignerinfo;
        }
        else
        {
            return null;
        }
    }

    private ObjectIdentifier contentType;
    private BigInt version;
    private AlgorithmId digestAlgorithmIds[];
    private ContentInfo contentInfo;
    private X509Certificate certificates[];
    private X509CRL crls[];
    private SignerInfo signerInfos[];
    private boolean oldStyle;
}
