package digsec.set.core.pkcs;

import java.io.*;
import java.util.Collection;
import java.util.Hashtable;
import sun.security.util.*;

// Referenced classes of package digsec.set.core.pkcs:
//            PKCS9Attribute

public class PKCS9Attributes
{

    public PKCS9Attributes(DerInputStream derinputstream)
        throws IOException
    {
        attributes = new Hashtable(3);
        derEncoding = decode(derinputstream);
        permittedAttributes = null;
    }

    public PKCS9Attributes(PKCS9Attribute apkcs9attribute[])
        throws IllegalArgumentException, IOException
    {
        attributes = new Hashtable(3);
        for(int i = 0; i < apkcs9attribute.length; i++)
        {
            ObjectIdentifier objectidentifier = apkcs9attribute[i].getOID();
            if(attributes.containsKey(objectidentifier))
                throw new IllegalArgumentException("PKCSAttribute " + apkcs9attribute[i].getOID() + " duplicated while constructing " + "PKCS9Attributes.");
            attributes.put(objectidentifier, apkcs9attribute[i]);
        }

        derEncoding = generateDerEncoding();
        permittedAttributes = null;
    }

    public PKCS9Attributes(ObjectIdentifier aobjectidentifier[], DerInputStream derinputstream)
        throws IOException
    {
        attributes = new Hashtable(3);
        if(aobjectidentifier != null)
        {
            permittedAttributes = new Hashtable(aobjectidentifier.length);
            for(int i = 0; i < aobjectidentifier.length; i++)
                permittedAttributes.put(aobjectidentifier[i], aobjectidentifier[i]);

        }
        else
        {
            permittedAttributes = null;
        }
        derEncoding = decode(derinputstream);
    }

    static DerEncoder[] castToDerEncoder(Object aobj[])
    {
        DerEncoder aderencoder[] = new DerEncoder[aobj.length];
        for(int i = 0; i < aderencoder.length; i++)
            aderencoder[i] = (DerEncoder)aobj[i];

        return aderencoder;
    }

    private byte[] decode(DerInputStream derinputstream)
        throws IOException
    {
        DerValue dervalue = derinputstream.getDerValue();
        byte abyte0[] = dervalue.toByteArray();
        abyte0[0] = 49;
        DerInputStream derinputstream1 = new DerInputStream(abyte0);
        DerValue adervalue[] = derinputstream1.getSet(3, true);
        for(int i = 0; i < adervalue.length; i++)
        {
            PKCS9Attribute pkcs9attribute = new PKCS9Attribute(adervalue[i]);
            ObjectIdentifier objectidentifier = pkcs9attribute.getOID();
            if(attributes.get(objectidentifier) != null)
                throw new IOException("Duplicate PKCS9 attribute: " + objectidentifier);
            if(permittedAttributes != null && !permittedAttributes.containsKey(objectidentifier))
                throw new IOException("Attribute " + objectidentifier + " not permitted in this attribute set");
            attributes.put(objectidentifier, pkcs9attribute);
        }

        return abyte0;
    }

    public void encode(byte byte0, OutputStream outputstream)
        throws IOException
    {
        outputstream.write(byte0);
        outputstream.write(derEncoding, 1, derEncoding.length - 1);
    }

    private byte[] generateDerEncoding()
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        Object aobj[] = attributes.values().toArray();
        deroutputstream.putOrderedSetOf((byte)49, castToDerEncoder(aobj));
        return deroutputstream.toByteArray();
    }

    public PKCS9Attribute getAttribute(String s)
    {
        return (PKCS9Attribute)attributes.get(PKCS9Attribute.getOID(s));
    }

    public PKCS9Attribute getAttribute(ObjectIdentifier objectidentifier)
    {
        return (PKCS9Attribute)attributes.get(objectidentifier);
    }

    public Object getAttributeValue(String s)
        throws IOException
    {
        ObjectIdentifier objectidentifier = PKCS9Attribute.getOID(s);
        if(objectidentifier == null)
            throw new IOException("Attribute name " + s + " not recognized or not supported.");
        else
            return getAttributeValue(objectidentifier);
    }

    public Object getAttributeValue(ObjectIdentifier objectidentifier)
        throws IOException
    {
        try
        {
            Object obj = getAttribute(objectidentifier).getValue();
            return obj;
        }
        catch(NullPointerException _ex)
        {
            throw new IOException("No value found for attribute " + objectidentifier);
        }
    }

    public PKCS9Attribute[] getAttributes()
    {
        PKCS9Attribute apkcs9attribute[] = new PKCS9Attribute[attributes.size()];
        int i = 0;
        for(int j = 1; j < PKCS9Attribute.PKCS9_OIDS.length && i < apkcs9attribute.length; j++)
        {
            apkcs9attribute[i] = getAttribute(PKCS9Attribute.PKCS9_OIDS[j]);
            if(apkcs9attribute[i] != null)
                i++;
        }

        return apkcs9attribute;
    }

    public byte[] getDerEncoding()
        throws IOException
    {
        return (byte[])derEncoding.clone();
    }

    public String toString()
    {
        StringBuffer stringbuffer = new StringBuffer(200);
        stringbuffer.append("PKCS9 Attributes: [\n\t");
        boolean flag = true;
        for(int i = 1; i < PKCS9Attribute.PKCS9_OIDS.length; i++)
        {
            PKCS9Attribute pkcs9attribute = getAttribute(PKCS9Attribute.PKCS9_OIDS[i]);
            if(pkcs9attribute != null)
            {
                if(flag)
                    flag = false;
                else
                    stringbuffer.append(";\n\t");
                stringbuffer.append(pkcs9attribute.toString());
            }
        }

        stringbuffer.append("\n\t] (end PKCS9 Attributes)");
        return stringbuffer.toString();
    }

    private final Hashtable attributes;
    private final Hashtable permittedAttributes;
    private final byte derEncoding[];
}
