package digsec.set.core.pkcs;
import java.io.*;
import sun.misc.HexDumpEncoder;
import sun.security.util.*;
import sun.security.x509.X500Name;

public class RecipientInfo
  implements DerEncoder
{

 /**
  * Construct this object from a DER encoded input stream.
  *
  * @param	din the DER encoded input stream.
  */
  public RecipientInfo(DerInputStream din)
    throws IOException, ParsingException
  {
    // Get the version
    version = new BigInt(din.getInteger().toInt());

    DerValue dv[] = din.getSequence(2);
    byte data[] = dv[0].toByteArray();

    // Get the issuerName
    issuerName = new X500Name(new DerValue((byte)48, data));
    // And get the serialNumber
    certificateSerialNumber = dv[1].getInteger();

    // Get the keyEncryptionAlgorithm
    DerValue dvAlgId = din.getDerValue();
    keyEncryptionAlgorithm = AlgorithmId.parse(dvAlgId);

    // Get the encryptedKey
    encryptedKey = din.getOctetString();

    // Validates the input
    if (din.available() != 0)
      throw new ParsingException("extra data at the end");
    else
      return;

  }

 /**
  *
  */
  public RecipientInfo(X500Name issuerDN, BigInt ser,
                       AlgorithmId keyAlg, byte key[])
  {
    version = new BigInt(0);
    issuerName = issuerDN;
    certificateSerialNumber = ser;
    keyEncryptionAlgorithm = keyAlg;
    encryptedKey = key;
  }

 /**
  * Write this object to an output stream in DER encoded format.
  *
  * @param	out the output stream.
  */
  public void derEncode(OutputStream out)
    throws IOException
  {
    // Prepare the output stream
    DerOutputStream dos = new DerOutputStream();

    // Write the version
    dos.putInteger(version);

    // Write the issuerAndSerialNumber
    DerOutputStream dosIssuer = new DerOutputStream();
    issuerName.encode(dosIssuer);
    dosIssuer.putInteger(certificateSerialNumber);
    dos.write((byte)48, dosIssuer);

    // Write the keyEncryptionAlgorithm
    keyEncryptionAlgorithm.encode(dos);

    // Write the encryptedKey
    dos.putOctetString(encryptedKey);

    // Write to the outputstream
    DerOutputStream dout = new DerOutputStream();
    dout.write((byte)48, dos);
    out.write(dout.toByteArray());
  }

 /**
  * Write this object to an output stream.
  *
  * @param	dos the output stream.
  */
  public void encode(DerOutputStream dos)
    throws IOException
  {
    derEncode(dos);
  }

 /**
  * Get the certificateSerialNumber.
  *
  * @return	the certificateSerialNumber.
  */
  public BigInt getCertificateSerialNumber()
  {
    return certificateSerialNumber;
  }

 /**
  * Get the keyEncryptionAlgorithm.
  *
  * @return	the keyEncryptionAlgorithm.
  */
  public AlgorithmId getKeyEncryptionAlgorithm()
  {
    return keyEncryptionAlgorithm;
  }

 /**
  * Get the encryptedKey.
  *
  * @return	the encryptedKey.
  */
  public byte[] getEncryptedKey()
  {
    return encryptedKey;
  }

 /**
  * Get the issuer distinguished name.
  *
  * @return	the issuerName.
  */
  public X500Name getIssuerName()
  {
    return issuerName;
  }

 /**
  * Get the version of this recipient info object.
  *
  * @return	the version.
  */
  public BigInt getVersion()
  {
    return version;
  }

 /**
  * Return a string representation of this recipient info.
  *
  * @return	the string representation.
  */
  public String toString()
  {
    HexDumpEncoder hde = new HexDumpEncoder();
    String s = "";
    s = s + "Recipient Info for (issuer): " + issuerName + "\n";
    s = s + "\tversion: " + version + "\n";
    s = s + "\tcertificateSerialNumber: " + certificateSerialNumber + "\n";
    s = s + "\tkeyEncryptionAlgorithm: " + keyEncryptionAlgorithm + "\n";
    s = s + "\tencryptedKey: \n" + hde.encodeBuffer(encryptedKey) + "\n";
    return s;
  }

  private BigInt version;
  private X500Name issuerName;
  private BigInt certificateSerialNumber;
  private AlgorithmId keyEncryptionAlgorithm;
  private byte encryptedKey[];

  public static void main(String[] args) {
    try {	
	X500Name name = new X500Name("Arif", "DIGSEC", "Fasilkom", "Indonesia");
	BigInt version = new BigInt(1);
	AlgorithmId alg = new AlgorithmId(AlgorithmId.RSAEncryption_oid);
	byte[] encrKey = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};

	RecipientInfo rInfo = new RecipientInfo(name,version,alg,encrKey);
	DerOutputStream dout = new DerOutputStream();
	rInfo.encode(dout);
 	HexDumpEncoder hde = new HexDumpEncoder();
	System.out.println(hde.encodeBuffer(dout.toByteArray()));

	DerValue dv = new DerValue(dout.toByteArray());
	rInfo = new RecipientInfo(dv.toDerInputStream());
	/*
	DerInputStream din = new DerInputStream(dout.toByteArray());
	DerValue[] adv = din.getSequence(4);	
	rInfo = new RecipientInfo(new X500Name(adv[1]), new BigInt(123456), 
                                AlgorithmId.parse(adv[2]),
					  adv[3].toByteArray());
      */
	System.out.println(rInfo.toString());	
    }
    catch(Exception e) {
	e.printStackTrace();
    }	
  }

}
