package digsec.set.payment.component;

import sun.security.util.*;
import java.io.*;
import java.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;

/**
 *	<pre>	
 *	1800 AuthTokenData ::= SEQUENCE {
 *	1801    transIDs          TransIDs,
 *	1802    purchAmt          CurrencyAmount,
 *	1803    merchantID        MerchantID,
 *	1804    acqBackKeyData    BackKeyData  OPTIONAL,
 *	1805    installRecurData  [0] InstallRecurData  OPTIONAL,
 *	1806    recurringCount    [1] INTEGER (1..MAX)  OPTIONAL,
 *	1807    prevAuthDateTime  Date,
 *	1808    totalAuthAmount   [2] CurrencyAmount  OPTIONAL,
 *	1809    authTokenOpaque   [3] EXPLICIT TokenOpaque OPTIONAL
 *	1810 }
 *	</pre>
 */

public class AuthTokenData implements DerEncoder {

	//fields
	TransIDs 		transIDs;
	CurrencyAmount 	purchAmt;
	String		merchantID;
	//BackKeyData	acqBackKeyData; //this hasn't been implemented
	InstallRecurData 	installRecurData; //implicit context-specific tag [0]
	BigInt 		recurringCount; //implicit context-specific tag [1]
	Date			prevAuthDateTime; 
	CurrencyAmount	totalAuthAmount; //implicit context-specific tag [2]
	//TokenOpaque	authTokenOpaque; //explicit context-specific tag [3], hasn't been implemented


	//Constructor
	public AuthTokenData(TransIDs 		transIDs,
				   CurrencyAmount 	purchAmt,
				   String			merchantID,
				   InstallRecurData 	installRecurData,	
				   BigInt			recurringCount,
				   Date			prevAuthDateTime,
				   CurrencyAmount		totalAuthAmount) throws Exception {

		this.transIDs		= transIDs;
		this.purchAmt		= purchAmt;
		this.merchantID		= merchantID;
		this.installRecurData	= installRecurData;	
		this.recurringCount	= recurringCount;
		this.prevAuthDateTime	= prevAuthDateTime;
		this.totalAuthAmount	= totalAuthAmount;	
	}

	public AuthTokenData(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(9);
		//set transIDs
		transIDs = new TransIDs(new DerInputStream(adv[0].toByteArray()));
		//set purchAmt
		purchAmt = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
		//set merchantID
		merchantID = adv[2].getPrintableString();
		//set acqBackKeyData - optional
			
		//set installRecurData - optional - [0]
		if(adv[4].tag == (byte)-96) {
			adv[4].tag = (byte)48;
			installRecurData = new InstallRecurData(new DerInputStream(adv[4].toByteArray()));
		}
		else
			installRecurData=null;
		
		//set recurringCount - optional - [1]
		if(adv[5].tag == (byte)-127) {
			adv[5].tag = (byte)0x02;
			recurringCount = adv[5].getInteger();
		}
		else
			recurringCount = null;	

		//set prevAuthDateTime
		prevAuthDateTime = (new DerInputStream(adv[6].toByteArray())).getGeneralizedTime();

		//set totalAuthAmount - optional - [2]
		if(adv[7].tag == (byte)-94) {
			adv[7].tag = (byte)48;
			totalAuthAmount = new CurrencyAmount(new DerInputStream(adv[7].toByteArray()));
		}
		else
			totalAuthAmount = null;

		//set authTokenOpaque
		
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout1;
		//write transIDs
		transIDs.derEncode(dout);				
		//write purchAmt
		purchAmt.derEncode(dout);				
		//write merchantID	
		dout.putPrintableString(merchantID);		
		//write acqBackKeyData - optional
		dout.putNull();
		
		//write installRecurData - optional - [0]
		if(installRecurData!=null) {
			dout1 = new DerOutputStream();
			installRecurData.derEncode(dout1);
			dout.writeImplicit((byte)-96,dout1); 
		}
		else
			dout.putNull();
		
		//write RecurringCount - optional - [1]
		if(recurringCount!=null) {
			dout1 = new DerOutputStream();
			dout1.putInteger(recurringCount);
			dout.writeImplicit((byte)-127,dout1);
		}
		else
			dout.putNull();
		
		//write prevAuthDateTime
		dout.putGeneralizedTime(prevAuthDateTime);
	
		//write totalAuthAmount - optional - [2]
		if(totalAuthAmount!=null) {
			dout1 = new DerOutputStream();
			totalAuthAmount.derEncode(dout1);
			dout.writeImplicit((byte)-94,dout1);
		}
		else 
			dout.putNull();

		//write authTokenOpaque - optional - [3]
		dout.putNull();
		
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());			
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthTokenData>\n";
		s += "transIDs " + transIDs + "\n";
		s += "purchAmt " + purchAmt + "\n";
		s += "merchantID " + merchantID + "\n";
		if(installRecurData!=null)
			s += "installRecurData " + installRecurData + "\n";
		if(recurringCount!=null)
			s += "recurringCount " + recurringCount + "\n";
		s += "prevAuthDateTime " + prevAuthDateTime + "\n";
		if(totalAuthAmount!=null)
			s += "totalAuthAmount " + totalAuthAmount + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
			System.out.println(auth);
			DerOutputStream dout = new DerOutputStream();
			auth.encode(dout);			
			auth = new AuthTokenData(new DerInputStream(dout.toByteArray()));
			System.out.println("-----------after encode-------------");
			System.out.println(auth);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
		
}