package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.lang.Math;	
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 * <pre>
 * 1843 CurrencyAmount ::= SEQUENCE {
 * 1844    currency  Currency, -- Currency code as defined in ISO-4217
 * 1845    amount    INTEGER (0..MAX),
 * 1846    amtExp10  INTEGER (MIN..MAX)
 * 1847                        -- Base ten exponent, such that the value in local
 * 1848                        -- currency is "amount * (10 ** amtExp10)"
 * 1849                        -- The exponent shall be the same value as defined
 * 1850                        -- for the minor unit of currency in ISO-4217.
 * 1851 }
 *
 * 263 Currency ::= INTEGER (1..999)        -- ISO-4217 currency code
 * </pre>
 */
public class CurrencyAmount implements DerEncoder 
{
	private BigInt currency;
	private BigInt amount;
	private BigInt amtExp10;

	/**
	 * Constructor for CurrencyAmount
	 *
	 * @param cur currency code as in ISO-4217
	 * @param amnt amount value
	 * @param exp base ten exponent
	 *
	 */
	public CurrencyAmount(BigInt cur, BigInt amnt, BigInt exp) throws Exception {
		currency = cur;
		amount = amnt;
		amtExp10 = exp;
	}

	/**
	 * Constructor for CurrencyAmount
	 *
	 * @param cur currency code as in ISO-4217
	 * @param amnt amount value
	 *
	 */
	public CurrencyAmount(BigInt cur, float amnt) throws Exception {
		currency = cur;
		float temp = amnt;
		int expTemp=0;

		System.out.println("float: " + temp);
		while((temp%10)!=0) {
			temp = temp * 10;	
			expTemp--;
			System.out.println(temp + " " + expTemp);
		}
		amount = new BigInt((int) temp);
		BigInteger b = (new BigInt(Math.abs(expTemp))).toBigInteger();
		amtExp10 = new BigInt(b.negate());	
	}

	/**
	 * Constructor for CurrencyAmount
	 *
	 * @param din DER representation of CurrencyAmount
	 *
	 */
	public CurrencyAmount(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(3);
		//isi currency
		currency = new BigInt(adv[0].getInteger().toInt());

		//isi amount
		amount = new BigInt(adv[1].getInteger().toInt());

		//isi revision
		amtExp10 = new BigInt(adv[2].getInteger().toInt());
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(currency);			//tulis currency		
		dout.putInteger(amount);			//tulis amount
		dout.putInteger(amtExp10);			//tulis amtExp10
		
		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for getting the currency code
	 *
	 * @return currency code
	 */
	public BigInt getCurrency() {
		return currency;
	}

	/**
	 * Method for getting the amount
	 *
	 * @return amount of purchase
	 */
	public BigInt getAmount() {
		return amount;
	}

	/**
	 * Method for getting the amtExp10
	 *
	 * @return base 10 exponent 
	 */
	public BigInt getAmtExp10() {
		return amtExp10;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "CurrencyAmount ::= Currency: " + currency.toInt() + "\n";
		s = s + "CurrencyAmount ::= Amount: " + amount.toInt() + "\n";
		s = s + "CurrencyAmount ::= Exponent: " + amtExp10.toInt() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			//CurrencyAmount cur = new CurrencyAmount(new BigInt(120), (float) 15.5);
			System.out.println(cur);

			DerOutputStream dout = new DerOutputStream();
			cur.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			cur = new CurrencyAmount(new DerInputStream(dout.toByteArray()));
			System.out.println(cur);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

