package digsec.set.payment.component;

import java.io.*;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;

/**
 * <pre>
 * 872 HODInput ::= SEQUENCE {
 * 873    od                OD,
 * 874    purchAmt          CurrencyAmount,
 * 875    odSalt            Nonce,
 * 876    installRecurData  [0] InstallRecurData  OPTIONAL,
 * 877    odExtensions      [1] MsgExtensions {{ODExtensionsIOS}} OPTIONAL
 * 878 }
 *
 * 882 OD ::= OCTET STRING                                 -- Order description
 * 
 * 296 Nonce ::= OCTET STRING (SIZE(20))
 * </pre>
 */
public class HODInput implements DerEncoder 
{
	public byte[] od;
	public CurrencyAmount purchAmt;
	public byte[] odSalt;
	public InstallRecurData installRecurData;
	public static ObjectIdentifier id_set_content_HODInput = new ObjectIdentifier("2.23.42.0.7");
	
	/**
	 * Constructor for HODInput
	 *
	 * @param order order description
	 * @param pAmnt purchase amount
	 *
	 */
	public HODInput(byte[] od, CurrencyAmount purchAmt, byte[] odSalt) throws Exception {
		this.od = od;
		this.purchAmt = purchAmt;
		this.odSalt = odSalt;
	}

	/**
	 * Constructor for HODInput
	 *
	 * @param order order description
	 * @param pAmnt purchase amount
	 * @param insRecurData install recurring data
	 *
	 */
	public HODInput(byte[] od, CurrencyAmount purchAmt, byte[] odSalt, InstallRecurData installRecurData) throws Exception {
		this.od = od;
		this.purchAmt = purchAmt;
		this.odSalt = odSalt;
		this.installRecurData = installRecurData;
	}

	/**
	 * Constructor for HODInput
	 *
	 * @param din DER representation of HODInput
	 *
	 */
	public HODInput(DerInputStream din) throws Exception {
		//jadiin Content Info
		ContentInfo contentInfo = new ContentInfo(din);
		//ambil content
		DerValue dv = contentInfo.getContent();
		DerInputStream din1 = new DerInputStream(dv.toByteArray());
		//ambil sequence
		DerValue[] adv = din1.getSequence(5);

		//ambil order description
		od = adv[0].getOctetString();

		//ambil purchase amount
		purchAmt = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));

		//ambil salt
		odSalt = adv[2].getOctetString();

		//ambil install recur data	
		if (adv[3].tag==(byte)0xA0) {
			adv[3].resetTag(DerValue.tag_Sequence);
			installRecurData = new InstallRecurData(new DerInputStream(adv[3].toByteArray()));
		}
		else
			installRecurData = null;
		
		//ambil extension
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putOctetString(od);			//tulis order description		
		purchAmt.encode(dout);				//tulis purchase amount	
		dout.putOctetString(odSalt);			//tulis salt

		//kalau installRecurData ada, tulis, kalau tidak, tulis null
		if (installRecurData != null) {
			DerOutputStream tempdout = new DerOutputStream();
			installRecurData.derEncode(tempdout);
			DerValue tempder = new DerValue(tempdout.toByteArray());
			tempder.resetTag((byte)0xA0);
			dout.putDerValue(tempder);
		}
		else
			dout.putNull();
		
		//di sini mestinya ditulis msgExtension
			dout.putNull();
		
		//tambahin tag sequence
		DerValue dv = new DerValue(DerValue.tag_Sequence, dout.toByteArray());
		//jadiin ContentInfo
		ContentInfo cInfo = new ContentInfo(id_set_content_HODInput,dv);		
		cInfo.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for getting order description
	 *
	 * @return order description
	 */
	public byte[] getOd() {
		return od;
	}

	/**
	 * Method for getting purchase amount
	 *
	 * @return purchase amount 
	 */
	public CurrencyAmount getPurchAmt() {
		return purchAmt;
	}

	/**
	 * Method for getting order description salt
	 *
	 * @return order description salt
	 */
	public byte[] getOdSalt() {
		return odSalt;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "HODInput ::= Order Description: " + "\n" + hde.encodeBuffer(od) + "\n";

		s = s + "HODInput ::= Purchase Amount: " + "\n" + purchAmt.toString() + "\n";
		s = s + "HODInput ::= odSalt: " + "\n" + hde.encodeBuffer(odSalt) + "\n";

		if(installRecurData != null) 
			s = s + "HODInput ::= InstallRecurData: " + "\n" + installRecurData.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount purchAmt = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,purchAmt,odSalt,install);  
			System.out.println(hodi);

			DerOutputStream dout = new DerOutputStream();
			hodi.encode(dout);

			FileOutputStream fo = new FileOutputStream("HODInput.dat");
			fo.write(dout.toByteArray());
			fo.close();

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			FileInputStream fi = new FileInputStream("HODInput.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);
			hodi = new HODInput(new DerInputStream(input));
			System.out.println(hodi);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

