package digsec.set.payment.component;

import java.util.Date;
import java.util.Calendar;
import java.io.*;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;
import java.security.SecureRandom;

/**
 * <pre>
 * 853 OIData ::= SEQUENCE {                          -- Order Information Data
 * 854    transIDs      TransIDs,
 * 855    rrpid         RRPID,
 * 856    chall-C       Challenge,
 * 857    hod           HOD,
 * 858    odSalt        Nonce,
 * 859    chall-M       Challenge  OPTIONAL,
 * 860    brandID       BrandID,
 * 861    bin           BIN,
 * 862    odExtOIDs     [0] OIDList  OPTIONAL,
 * 863    oiExtensions  [1] MsgExtensions {{OIExtensionsIOS}} OPTIONAL
 * 864 }
 *
 * 324 RRPID ::= OCTET STRING(SIZE(20)) -- Request response pair identification
 *
 * 259 Challenge ::= OCTET STRING (SIZE(20))   -- Signature freshness challenge
 *
 * 296 Nonce ::= OCTET STRING (SIZE(20))
 *
 * 232 BrandID ::= SETString { ub-BrandID }
 *
 * 250 BIN ::= NumericString (SIZE(6))            -- Bank identification number
 * 
 * 868 OIDList ::= SEQUENCE OF OBJECT IDENTIFIER
 * </pre>
 */
public class OIData implements DerEncoder 
{
	public TransIDs transIDs;
	public byte[] rrpid;
	public byte[] challC;
	public HOD hod;
	public byte[] odSalt;
	public byte[] challM;
	public String brandID;
	public String bin;
	public ObjectIdentifier[] OIDList;
	public static ObjectIdentifier id_set_content_OIData = new ObjectIdentifier("2.23.42.0.3");

	/**
	 * Constructor for OIData
	 *
	 * @param transIDs transaction ID
	 * @param rrpid 
	 * @param challC
	 * @param hod
	 * @param odSalt
	 * @param challM (optional)
	 * @param brandID
	 * @param bin
	 * @param OIDList
	 *
	 */
	public OIData(
			TransIDs transIDs, 
			byte[] rrpid,
			byte[] challC, 
			HOD hod, 
			byte[] odSalt,
			byte[] challM,
			String brandID,
			String bin) throws Exception {

		this.transIDs = transIDs;
		this.rrpid = rrpid;
		this.challC = challC;
		this.hod = hod;
		this.odSalt = odSalt;
		this.challM = challM;
		this.brandID = brandID;
		this.bin = bin;
	}

	/**
	 * Constructor for Inputs
	 *
	 * @param din DER representation of HODInput
	 *
	 */
	public OIData(DerInputStream din) throws Exception {
		//jadiin Content Info
		ContentInfo contentInfo = new ContentInfo(din);
		//ambil content
		DerValue dv = contentInfo.getContent();
		DerInputStream din1 = new DerInputStream(dv.toByteArray());
		
		DerValue[] adv = din1.getSequence(8);

		this.transIDs = new TransIDs(new DerInputStream(adv[0].toByteArray()));
		this.rrpid = adv[1].getOctetString();
		this.challC = adv[2].getOctetString();
		this.hod = new HOD(new DerInputStream(adv[3].toByteArray()));
		this.odSalt = adv[4].getOctetString();

		//isi challM
		if (adv[5].tag!=(byte)0x05)
			this.challM = adv[5].getOctetString();
		else
			challM = null;

		this.brandID = adv[6].getPrintableString();
		this.bin = adv[7].getPrintableString();

		//di sini mestinya ambil OIDList

		//di sini mestinya ambil MsgExtensions

	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		transIDs.derEncode(dout);					
		dout.putOctetString(rrpid);				
		dout.putOctetString(challC);
		hod.encode(dout);
		dout.putOctetString(odSalt);
		
		//kalau challM ada, tulis, kalau tidak, tulis null
		if (challM != null) 
			dout.putOctetString(challM);
		else
			dout.putNull();

		dout.putPrintableString(brandID);
		dout.putPrintableString(bin);

		//tambahin tag sequence
		DerValue dv = new DerValue(DerValue.tag_Sequence, dout.toByteArray());
	
		//jadiin ContentInfo
		ContentInfo cInfo = new ContentInfo(id_set_content_OIData,dv);		
		cInfo.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "<OIData>\n";
		s = s + "OIData ::= transIDs: " + "\n" + transIDs.toString() + "\n";
		s = s + "OIData ::= rrpid: " + "\n" + hde.encode(rrpid) + "\n";
		s = s + "OIData ::= challC: " + "\n" + hde.encode(challC) + "\n";
		s = s + "OIData ::= hod: " + "\n" + hod.toString() + "\n";
		s = s + "OIData ::= odSalt: " + "\n" + hde.encode(odSalt) + "\n";
		if(challM!=null)
			s = s + "OIData ::= challM: " + "\n" + hde.encode(challM) + "\n";
		s = s + "OIData ::= brandID: " +  brandID + "\n";
		s = s + "OIData ::= bin: " +  bin + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			byte[] lid_M = {2,1,0,3,1,7};
			Calendar cal = Calendar.getInstance();
			cal.set(1999,1,14);
			Date pReqDate = cal.getTime();
			SecureRandom sr = new SecureRandom();	
			byte[] XID = new byte[20];
			sr.nextBytes(XID);

			TransIDs transIDs = new TransIDs(lid_C,lid_M, XID, pReqDate, null, "English");
			
			//*******bikin RRPID*********
			byte[] rrpid = {99,7,31,11,30,35,0,0,0,0,0,0,0,0,0,0,0,0,0,1};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			FileInputStream fi = new FileInputStream("HODInput.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);			

			HODInput hodi = new HODInput(new DerInputStream(input));  

			HOD hod = new HOD(hodi);

			//*******bikin BrandID*********
			String brandID = "Visa International";

			//*******bikin BIN*********
			String bin = "08123170";

			//******baru deh bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod,hodi.odSalt,challC,brandID,bin);

			System.out.println(oidata);

			DerOutputStream dout = new DerOutputStream();
			oidata.encode(dout);

			FileOutputStream fo = new FileOutputStream("OIData.dat");
			fo.write(dout.toByteArray());
			fo.close();

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");

			fi = new FileInputStream("OIData.dat");
			input = new byte[fi.available()];
			fi.read(input);
			fi.close();
			oidata = new OIData(new DerInputStream(input));
			System.out.println(oidata);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

