package digsec.set.payment.component;
import digsec.set.core.pkcs.*;
import digsec.set.core.crypto.OAEPEncoder;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.util.StringTokenizer;

/**
 *	<pre>
 *	300 PANData ::= SEQUENCE {
 *	301    pan         PAN,
 *	302    cardExpiry  CardExpiry,
 *	303    panSecret   Secret,
 *	304    exNonce     Nonce
 *	305 }
 *	
 *	298 PAN ::= NumericString (SIZE(1..19))
 *	
 *	252 CardExpiry ::= NumericString (SIZE(6)) -- YYYYMM expiration date of card
 *	
 *	296 Nonce ::= OCTET STRING (SIZE(20))
 *	
 *        Secret ::= OCTET STRING (SIZE(1..20))
 */
 
public class PANData implements DerEncoder, OAEPEncoder {

	protected String pan; // size(1..19)
	protected String cardExpiry; //size(6)
	protected byte[] secret; // size (1..20)
	protected byte[] exNonce; // size(20)	

	public static ObjectIdentifier id_set_content_PANData = new ObjectIdentifier("2.23.42.0.0");

	public PANData(String pan, String exp, byte[] secret, byte[] nonce) throws Exception{
		if(pan.length()<1 || pan.length()>19 
               || nonce.length != 20 || exp.length()!=6
               || secret.length<1 || secret.length>20)  
			throw new RuntimeException("Invalid input of PANOnly");
		this.pan = pan;
		cardExpiry = exp;
		this.secret = secret;
		exNonce = nonce;
	}
	
	public PANData(DerInputStream din) throws Exception {

		ContentInfo cInfo = new ContentInfo(din);
		DerInputStream din1 = new DerInputStream(cInfo.getContent().toByteArray());

		DerValue[] adv = din1.getSequence(4);
		pan = adv[0].getPrintableString();
		cardExpiry = adv[1].getPrintableString();
		secret = adv[2].getOctetString();
		exNonce = adv[3].getOctetString();
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		DerValue[] adv = new DerValue[4];
		adv[0] = new DerValue(pan); //18 is tag for NumericString 
		adv[1] = new DerValue(cardExpiry);
		adv[2] = new DerValue((byte)4,secret);
		adv[3] = new DerValue((byte)4,exNonce); 	
		dout.putSequence(adv);
		ContentInfo cInfo = new ContentInfo(id_set_content_PANData, new DerValue(dout.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		
		derEncode(dout);
	}	

	public byte[] encodeOAEP () throws IOException{
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		
		StringBuffer strbuf = new StringBuffer(pan);
		if(pan.length() < 19) {
			for(int i=pan.length(); i<19; i++)
				strbuf=strbuf.append((char)0x20); 
		}
		bout.write(strbuf.toString().getBytes());
		bout.write(cardExpiry.getBytes());

		StringBuffer strbuf1 = new StringBuffer(new String(secret));
		if(secret.length < 20) {
			for(int i=secret.length; i<20; i++)
				strbuf1=strbuf1.append((char)0x00); 
		}
		bout.write(strbuf1.toString().getBytes());

		bout.write(exNonce);		
		return bout.toByteArray();
	}

	public static PANData parseOAEP(byte[] oaep) throws Exception {
		ByteArrayInputStream bin = new ByteArrayInputStream(oaep);
		byte[] b_pan = new byte[19];
		bin.read(b_pan,0,19);
		
		StringTokenizer strtok = new StringTokenizer(new String(b_pan));		
		String newPan = strtok.nextToken(" ");		

		byte[] b_exp = new byte[6];
		bin.read(b_exp,0,6);
		String newExp = new String(b_exp);

		byte[] b_secret = new byte[20];
		bin.read(b_secret,0,20);

		StringTokenizer strtok1 = new StringTokenizer(new String(b_secret));		
		String newSecret = strtok1.nextToken("\0");		
		
		byte[] nonce = new byte[20];
		bin.read(nonce,0,20);
		return new PANData(newPan, newExp, newSecret.getBytes(), nonce);
	}

	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();
		String s = "";
		s += "PANData\n";
		s += "pan: " + pan + "\n";
		s += "cardExpiry: " + cardExpiry + "\n";
		s += "panSecret: " + "\n" + hde.encodeBuffer(secret) + "\n";
		s += "exNonce: " + "\n" + hde.encodeBuffer(exNonce) + "\n";
		return s;
	}

	public String getPAN() {
		return pan;
	}

	public String getExpiry() {
		return cardExpiry;
	}

	public byte[] getSecret() {
		return secret;
	}

	public byte[] getNonce() {
		return exNonce;
	}

	public static void main (String[] args) {
		try { 
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			byte[] secret = {3,1,7,8,1,2};
			PANData pData = new PANData("1294", "199908", secret, ab);
			DerOutputStream dout = new DerOutputStream();
			pData.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			pData = new PANData(new DerInputStream(dout.toByteArray()));
			System.out.println(pData);
			byte[] temp = pData.encodeOAEP();
			System.out.println(hde.encodeBuffer(temp));
			pData = PANData.parseOAEP(temp);
			System.out.println(pData);
			dout.reset();
			pData.encode(dout);
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}
