package digsec.set.payment.component;
import digsec.set.core.pkcs.*;
import digsec.set.core.crypto.OAEPEncoder;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.util.StringTokenizer;

/**
 *	<pre>
 *	314 PANToken ::= SEQUENCE {
 *	315    pan         PAN,
 *	316    cardExpiry  CardExpiry,
 *	317    exNonce     Nonce
 *	318 }
 *	
 *	298 PAN ::= NumericString (SIZE(1..19)) 
 *	
 *	252 CardExpiry ::= NumericString (SIZE(6)) -- YYYYMM expiration date of card
 *	
 *	296 Nonce ::= OCTET STRING (SIZE(20))
 */

public class PANToken implements DerEncoder, OAEPEncoder {

	protected String pan; // size(1..19)
	protected String cardExpiry; //size(6)
	protected byte[] exNonce; // size(20)	

	public ObjectIdentifier id_set_content_PANToken = new ObjectIdentifier("2.23.42.0.1");

	public PANToken(String pan, String exp, byte[] nonce) throws Exception{
		if(pan.length()<1 || pan.length()>19 || nonce.length != 20 || exp.length()!=6)
			throw new RuntimeException("Invalid input of PANOnly");
		this.pan = pan;
		cardExpiry = exp;
		exNonce = nonce;
	}
	
	public PANToken(DerInputStream din) throws Exception {

		ContentInfo cInfo = new ContentInfo(din);
		DerInputStream din1 = new DerInputStream(cInfo.getContent().toByteArray());

		DerValue[] adv = din1.getSequence(3);
		pan = adv[0].getPrintableString();
		cardExpiry = adv[1].getPrintableString();
		exNonce = adv[2].getOctetString();
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		DerValue[] adv = new DerValue[3];
		adv[0] = new DerValue(pan); //18 is tag for NumericString 
		adv[1] = new DerValue(cardExpiry);
		adv[2] = new DerValue((byte)4,exNonce); 	
		dout.putSequence(adv);
		ContentInfo cInfo = new ContentInfo(id_set_content_PANToken, new DerValue(dout.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		
		derEncode(dout);
	}	

	public byte[] encodeOAEP () throws IOException{
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		
		StringBuffer strbuf = new StringBuffer(pan);
		if(pan.length() < 19) {
			for(int i=pan.length(); i<19; i++)
				strbuf=strbuf.append((char)0x20); 
		}
		bout.write(strbuf.toString().getBytes());
		bout.write(cardExpiry.getBytes());
		bout.write(exNonce);		
		return bout.toByteArray();
	}

	public static PANToken parseOAEP(byte[] oaep) throws Exception {
		ByteArrayInputStream bin = new ByteArrayInputStream(oaep);
		byte[] b_pan = new byte[19];
		bin.read(b_pan,0,19);
		
		StringTokenizer strtok = new StringTokenizer(new String(b_pan));		
		String newPan = strtok.nextToken(" ");		

		byte[] b_exp = new byte[6];
		bin.read(b_exp,0,6);
		String newExp = new String(b_exp);
		byte[] nonce = new byte[20];
		bin.read(nonce,0,20);
		return new PANToken(newPan, newExp, nonce);
	}

	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();
		String s = "";
		s += "PANToken\n";
		s += "pan: " + pan + "\n";
		s += "cardExpiry: " + cardExpiry + "\n";
		s += "exNonce: " + "\n" + hde.encodeBuffer(exNonce) + "\n";
		return s;
	}

	public String getPAN() {
		return pan;
	}

	public String getExpiry() {
		return cardExpiry;
	}

	public byte[] getNonce() {
		return exNonce;
	}

	public static void main (String[] args) {
		try { 
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);
			DerOutputStream dout = new DerOutputStream();
			pToken.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			pToken = new PANToken(new DerInputStream(dout.toByteArray()));
			System.out.println(pToken);
			byte[] temp = pToken.encodeOAEP();
			System.out.println(hde.encodeBuffer(temp));
			pToken = PANToken.parseOAEP(temp);
			System.out.println(pToken);
			dout.reset();
			pToken.encode(dout);
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}
