package digsec.set.payment.component;

import java.io.*;
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;
import java.security.*;
import au.net.aba.crypto.provider.*;
import cryptix.provider.Cryptix;

/**
 * <pre>
 * 833 PIHead ::= SEQUENCE {
 * 834    transIDs          TransIDs,
 * 835    inputs            Inputs,
 * 836    merchantID        MerchantID,
 * 837    installRecurData  [0] InstallRecurData  OPTIONAL,
 * 838    transStain        TransStain,
 * 839    swIdent           SWIdent,
 * 840    acqBackKeyData    [1] EXPLICIT BackKeyData  OPTIONAL,
 * 841    piExtensions      [2] MsgExtensions {{PIExtensionsIOS}} OPTIONAL
 * 842 }
 *
 * 294 MerchantID ::= SETString { ub-MerchantID }
 *
 * 851 TransStain ::= HMAC { XID, Secret }
 * 
 * 328 SWIdent ::= VisibleString (SIZE(1..ub-SWIdent))    -- Software identification
 * </pre>
 */
public class PIHead implements DerEncoder 
{
	public TransIDs transIDs;
	public Inputs inputs;
	public String merchantID;
	public InstallRecurData installRecurData;
	public HMAC transStain;
	public String swIdent;
	public BackKeyData acqBackKeyData;

	/**
	 * Constructor for PIHead
	 *
	 * @param transIDs transaction ID
	 * @param inputs inputs
	 * @param merchantID merchant identification
	 * @param installRecurData 
	 * @param transStain transaction stain : HMAC(XID,Secret)
	 * @param swIdent software identification
	 * @param acqBackKeyData back key data
	 *
	 */
	public PIHead(
			TransIDs transIDs, 
			Inputs inputs,
			String merchantID, 
			InstallRecurData installRecurData, 
			HMAC transStain,
			String swIdent,
			BackKeyData acqBackKeyData) throws Exception {

		this.transIDs = transIDs;
		this.inputs = inputs;
		this.merchantID = merchantID;
		this.installRecurData = installRecurData;
		this.transStain = transStain;
		this.swIdent = swIdent;
		this.acqBackKeyData = acqBackKeyData;
	}

	/**
	 * Constructor for PIHead
	 *
	 * @param transIDs transaction ID
	 * @param inputs inputs
	 * @param merchantID merchant identification
	 * @param installRecurData 
	 * @param XID id
	 * @param secret secret key
	 * @param swIdent software identification
	 * @param acqBackKeyData back key data
	 *
	 */
	public PIHead(
			TransIDs transIDs, 
			Inputs inputs, 
			String merchantID,
			InstallRecurData installRecurData, 
			byte[] XID,
			byte[] secret,
			String swIdent,
			BackKeyData acqBackKeyData) throws Exception {

		this.transIDs = transIDs;
		this.inputs = inputs;
		this.merchantID = merchantID;
		this.installRecurData = installRecurData;
		ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
					            new DerValue(DerValue.tag_OctetString,XID));		
		this.transStain = new HMAC(cInfo, secret);
		this.swIdent = swIdent;
		this.acqBackKeyData = acqBackKeyData;
	}

	/**
	 * Constructor for Inputs
	 *
	 * @param din DER representation of HODInput
	 *
	 */
	public PIHead(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(8);

		this.transIDs = new TransIDs(new DerInputStream(adv[0].toByteArray()));
		this.inputs = new Inputs(new DerInputStream(adv[1].toByteArray()));
		this.merchantID = adv[2].getPrintableString();

		//isi installRecurData
		if (adv[3].tag==(byte)0xA0) {
			adv[3].resetTag(DerValue.tag_Sequence);
			installRecurData = new InstallRecurData(new DerInputStream(adv[3].toByteArray()));
		}
		else
			installRecurData = null;

		this.transStain = new HMAC(new DerInputStream(adv[4].toByteArray()));
		this.swIdent = adv[5].getPrintableString();

		//isi acqBackKeyData
		if (adv[6].tag==(byte)0xA1)
			acqBackKeyData = new BackKeyData(adv[6].data);
		else
			acqBackKeyData = null;

		//di sini mestinya ambil PIExtension

	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		transIDs.derEncode(dout);					
		inputs.encode(dout);				
		dout.putPrintableString(merchantID);

		//kalau installRecurData ada, tulis, kalau tidak, tulis null
		if (installRecurData != null) {
			DerOutputStream midout = new DerOutputStream();
			installRecurData.derEncode(midout);
			dout.writeImplicit((byte)0xA0,midout);
		}
		else
			dout.putNull();

		transStain.encode(dout);
		dout.putPrintableString(swIdent);

		//kalau acqBackKeyData ada, tulis, kalau tidak, tulis null
		if (acqBackKeyData != null) {
			DerOutputStream midout1 = new DerOutputStream();
			acqBackKeyData.encode(midout1);
			dout.write((byte)0xA1,midout1);
		}
		else
			dout.putNull();

		//tulis PIExtension
		dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "PIHead ::= transIDs: " + "\n" + transIDs.toString() + "\n";
		s = s + "PIHead ::= inputs: " + "\n" + inputs.toString() + "\n";
		s = s + "PIHead ::= merchantID: " + "\n" + merchantID + "\n";
		if(installRecurData!=null)
			s = s + "PIHead ::= installRecurData: " + "\n" + installRecurData.toString() + "\n";
		s = s + "PIHead ::= transStain: " + "\n" + transStain.toString() + "\n";
		s = s + "PIHead ::= swIdent: " + "\n" + swIdent + "\n";
		if(acqBackKeyData!=null)
			s = s + "PIHead ::= acqBackKeyData: " + "\n" + acqBackKeyData.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			Security.addProvider(new ABAProvider());
			Security.addProvider(new Cryptix());

			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			byte[] lid_M = {2,1,0,3,1,7};
			Calendar cal = Calendar.getInstance();
			cal.set(1999,1,14);
			Date pReqDate = cal.getTime();
			SecureRandom sr = new SecureRandom();	
			byte[] XID = new byte[20];
			sr.nextBytes(XID);

			TransIDs transIDs = new TransIDs(lid_C,lid_M, XID, pReqDate, null, "English");

			//*******bikin inputs*********
			FileInputStream fi = new FileInputStream("HODInput.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);
			fi.close();
			HODInput hodi = new HODInput(new DerInputStream(input));  

			Inputs inputs = new Inputs(hodi,hodi.purchAmt);

			//*******bikin merchantID*********
			String merchantID = "Dhany's Butique";

			//*******bikin transStain*********
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******baru deh bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,hodi.installRecurData,transStain,swIdent,acqBackKeyData);

			System.out.println(piHead);

			DerOutputStream dout = new DerOutputStream();
			piHead.encode(dout);

			FileOutputStream fo = new FileOutputStream("PIHead.dat");
			fo.write(dout.toByteArray());
			fo.close();
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			fi = new FileInputStream("PIHead.dat");
			input = new byte[fi.available()];
			fi.read(input);
			fi.close();
			piHead = new PIHead(new DerInputStream(input));
			System.out.println(piHead);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

