package digsec.set.payment.component;

import sun.security.util.*;	
import java.util.Date;
import java.security.SecureRandom;
import sun.misc.HexDumpEncoder;  
import java.io.*;

/**
 *	<pre>
 *	337 TransIDs ::= SEQUENCE {
 *	338    lid-C      LocalID,
 *	339    lid-M      [0] LocalID  OPTIONAL,
 *	340    xid        XID,
 * 	341    pReqDate   Date,
 *	342    paySysID   [1] PaySysID  OPTIONAL,
 *	343    language   Language          -- Cardholder requested session language
 *	344 }
 *
 *	348 XID ::= OCTET STRING (SIZE(20))
 *
 *	320 PaySysID ::= VisibleString (SIZE(1..ub-paySysID=64))
 *
 *	282 Language ::= VisibleString (SIZE(1..ub-RFC1766-language=35))
 *
 *	284 LocalID ::= OCTET STRING (SIZE(1..20))
 *
 *	265 Date ::= GeneralizedTime
 *	</pre>
 */

public class TransIDs implements DerEncoder {

	//fields
	public byte[] 	lid_C;
	public byte[] 	lid_M;
	public byte[] 	xid;
	public Date 	pReqDate;
	public String	paySysId;
	public String	language="English";

	//Constructor
	public TransIDs(byte[] lid_C,
			    byte[] lid_M,
			    byte[] xid,	
			    Date   pReqDate,		
			    String paySysId,
			    String language) throws Exception {
		
		if(lid_C.length<1 || lid_C.length>20)
			throw new IllegalArgumentException("TransIDs::lid-C");
		this.lid_C=lid_C;

		if(lid_M!=null && (lid_M.length<1 || lid_M.length>20))
			throw new IllegalArgumentException("TransIDs::lid-M");
		this.lid_M=lid_M;

		if(xid!=null && xid.length != 20)
			throw new IllegalArgumentException("TransIDs::XID");
		this.xid = xid;

		this.pReqDate = pReqDate;

		this.paySysId=paySysId;

		this.language=language;
	}
	
	public TransIDs(DerInputStream din) throws Exception {

		din = din.getDerValue().toDerInputStream();

		lid_C=din.getOctetString();

		if((byte) din.peekByte() == (byte) 0x80) {
			DerValue dv1 = din.getDerValue();
			dv1.resetTag((byte)0x04);
			lid_M=dv1.getOctetString();
		}
		
		xid = din.getOctetString();
		pReqDate = din.getGeneralizedTime();

		if((byte) din.peekByte() == (byte) 0x81) {
			DerValue dv2 = din.getDerValue();
			dv2.resetTag((byte)0x13);
			paySysId = dv2.getPrintableString();
		}
				
		language = din.getDerValue().getPrintableString();
	}	

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout  = new DerOutputStream();
		DerOutputStream dout2 = new DerOutputStream();

		dout.putOctetString(lid_C);
		if(lid_M!=null) {
			dout2.putOctetString(lid_M);
			dout.writeImplicit((byte)0x80,dout2);
		}
		dout.putOctetString(xid);
		dout.putGeneralizedTime(pReqDate);
		if(paySysId!=null) {
			dout2.reset();
			dout2.putPrintableString(paySysId);
			dout.writeImplicit((byte)0x81,dout2);
		}
		dout.putPrintableString(language);
		
		dout2.reset();
		dout2.write((byte)0x30,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public static TransIDs getExample() throws Exception {
		
		byte[] lid_C = {8,1,2,3,1,7};
		byte[] lid_M = {2,1,0,3,1,7};

		byte[] seeds = {8,1,2,3,1,7};
		SecureRandom sr = new SecureRandom();
		byte[] xid = new byte[20];
		sr.nextBytes(xid);

		TransIDs id = new TransIDs(lid_C, lid_M, xid, new Date(), "Priharsanta Inc.", "Javanese");
		return id;
	}
	
  	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();
		String s = "<TransIDs>\n";
		s += "lid-C   :\n" + hde.encodeBuffer(lid_C) + "\n";
		if(lid_M!=null)
		s += "lid-M   :\n" + hde.encodeBuffer(lid_M) + "\n";
		s += "xid     :\n" + hde.encodeBuffer(xid) + "\n";
		s += "pReqDate:" + pReqDate + "\n";
		if(paySysId!=null)
		s += "paySysId:" + paySysId + "\n";
		s += "language:" + language + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			TransIDs id = TransIDs.getExample();
			System.out.println(id);
			DerOutputStream dout = new DerOutputStream();
			id.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			id = new TransIDs(new DerInputStream(dout.toByteArray()));
			System.out.println(id);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}		
}
