package digsec.set.payment.wrapper;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Random;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 * <pre>
 * 58 MessageHeader ::= SEQUENCE {
 * 59    version     INTEGER { setVer1(1) } (setVer1),
 * 60    revision    INTEGER (0) DEFAULT 0,   -- This is version 1.0
 * 61    date        Date,
 * 62    messageIDs  [0] MessageIDs  OPTIONAL,
 * 63    rrpid       [1] RRPID  OPTIONAL,
 * 64    swIdent     SWIdent
 * 65 }
 * </pre>
 */
public class MessageHeader implements DerEncoder 
{
	private BigInt version;
	private BigInt revision;
	private Date date;
	private MessageIDs messageIds;
	private byte[] rrpid;
	private String swIdent;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MessageHeader
	 *
	 * @param msgDate date and time of message generation
	 * @param mid ID of the SET message
	 * @param pid ID of response/request pair
	 * @param swident string identifying the software (vendor and version) initiating the request
	 *
	 */
	public MessageHeader(Date msgDate, MessageIDs mid, byte[] pid, String swident) throws Exception {
		version = new BigInt(1);
		revision = new BigInt(0);
		date = msgDate;
		messageIds = mid;
		rrpid = pid;
		swIdent = swident;
	}

	/**
	 * Constructor for MessageHeader
	 *
	 * @param din DER representation of MessageHeader
	 *
	 */
	public MessageHeader(DerInputStream din) throws IOException {
		din = din.getDerValue().toDerInputStream(); 
		//isi version
		version = new BigInt(din.getInteger().toInt());
		if (!version.equals(new BigInt(1)))
			throw new RuntimeException("Different version of MessageHeader");

		//isi revision
		revision = new BigInt(din.getInteger().toInt());

		//isi Date
		date = din.getGeneralizedTime(); 

		//isi MessageIds
		if ((byte) din.peekByte()==(byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag(DerValue.tag_Sequence);
			messageIds = new MessageIDs(new DerInputStream(dv.toByteArray()));
		}
		else
			messageIds = null;

		//isi RRPID
		if ((byte) din.peekByte()==(byte)0x81) {
			DerValue dv1 = din.getDerValue();
			dv1.resetTag(DerValue.tag_OctetString);
			rrpid = dv1.getOctetString();
		}
		else
			rrpid = null;

		//isi software identification
		swIdent = din.getDerValue().getPrintableString();	
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(version);			//tulis version		
		dout.putInteger(revision);			//tulis revision
		dout.putGeneralizedTime(date);		//tulis date
		
		//kalau messageIds ada, tulis, kalau tidak, biarin aja
		if (messageIds != null) {
			DerOutputStream midout = new DerOutputStream();
			messageIds.encode(midout);
			dout.write((byte)0xA0, midout);
		}
		
		//kalau RRPID ada, tulis, kalau tidak, biarin aja
		if (rrpid != null) {
			dout.putDerValue(new DerValue((byte)0x81,rrpid));	
		}
			
		dout.putPrintableString(swIdent);		//tulis swIdent

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the Version number
	 *
	 * @return version of SET messages: 1
	 */
	public BigInt getVersion() {
		return version;
	}

	/**
	 * Method for get the Revision number
	 *
	 * @return minor revision of SET messages: 0
	 */
	public BigInt getRevision() {
		return revision;
	}

	/**
	 * Method for get the Date
	 *
	 * @return date and time of message generation
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * Method for get the MessageIDs
	 *
	 * @return messageIDs of SET message
	 */
	public MessageIDs getMessageIDs() {
		return messageIds;
	}

	/**
	 * Method for get the RRPID
	 *
	 * @return ID of response/request pair
	 */
	public byte[] getRRPID() {
		return rrpid;
	}

	/**
	 * Method for get the SWIdent
	 *
	 * @return string identifying the software (vendor and version) initiating the request
	 */
	public String getSWIdent() {
		return swIdent;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "MessageHeader ::= Version: " + version.toInt() + "." + revision.toInt() + "\n";
		s = s + "MessageHeader ::= Date: " + date + "\n";
		if(messageIds!=null)
			s = s + "MessageHeader ::= MessageIDs: " + messageIds;
		if(rrpid!=null){
			HexDumpEncoder hde = new HexDumpEncoder();
        		s = s + "MessageHeader ::= RRPID \n" + hde.encodeBuffer(rrpid) + "\n";
		}
		s = s + "MessageHeader ::= SWIdent: " + swIdent + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			byte[] lidc = new byte[20];
			byte[] lidm = new byte[20];
			byte[] xid = new byte[20];
			byte[] rrpid = new byte[20];

			Random myRandom = new Random(5485076);
			myRandom.nextBytes(lidc);
			myRandom.setSeed(8642178);
			myRandom.nextBytes(lidm);
			myRandom.setSeed(7870481);
			myRandom.nextBytes(xid);
			myRandom.setSeed(5840502);
			myRandom.nextBytes(rrpid);

			MessageIDs mids = new MessageIDs(lidc,lidm,xid);

			Date date = new Date();
			String swident = "Bataknese";

			MessageHeader mh = new MessageHeader(date, mids, rrpid, swident);
			System.out.println(mh);

			DerOutputStream dout = new DerOutputStream();
			mh.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			MessageHeader mh2 = new MessageHeader(new DerInputStream(dout.toByteArray()));
			System.out.println(mh2);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

