package digsec.set.payment.wrapper;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Random;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 * <pre>
 * 67 MessageIDs ::= SEQUENCE {
 * 68    lid-C  [0] LocalID  OPTIONAL,
 * 69    lid-M  [1] LocalID  OPTIONAL,
 * 70    xID    [2] XID  OPTIONAL
 * 71 }
 *
 * 284 LocalID ::= OCTET STRING (SIZE(1..20))
 * 348 XID ::= OCTET STRING (SIZE(20))
 * </pre>
 */
public class MessageIDs implements DerEncoder 
{
	private byte[] lidc;
	private byte[] lidm;
	private byte[] xid;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MessageIDs
	 * Parameter can be either any of the IDs or null
	 *
	 * @param lidC Local ID; convenience label generated by and for Cardholder system
	 * @param lidM Local ID; convenience label generated by and for Merchant system 
	 * @param XID globally unique ID generated by Merchant in PInitRes, or by Cardholder in PReq
	 *
	 */
	public MessageIDs(byte[] lidC, byte[] lidM, byte[] XID) throws Exception {
		lidc = lidC;
		lidm = lidM;
		xid = XID;
	}

	/**
	 * Constructor for MessageIDs
	 *
	 * @param din DER representation of MessageIDs
	 */
	public MessageIDs(DerInputStream din) throws IOException {
		din = din.getDerValue().toDerInputStream();
		DerValue dv;
		//isi LID-C
		if ((byte) din.peekByte()==(byte)0x80) {
			dv = din.getDerValue();
			dv.resetTag((byte)0x04);
			lidc = dv.getOctetString();
		}
		else
			lidc = null;

		//isi LID-M
		if ((byte)din.peekByte()==(byte)0x81) {
			dv = din.getDerValue();
			dv.resetTag((byte)0x04);
			lidm = dv.getOctetString();
		}
		else
			lidm = null;

		//isi XID
		if ((byte)din.peekByte()==(byte)0x82) {
			dv = din.getDerValue();
			dv.resetTag((byte)0x04);
			xid = dv.getOctetString();
		}
		else
			xid = null;
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		//kalau LID-C ada, tulis, kalau tidak, biarin aja
		if (lidc != null)
			dout.putDerValue(new DerValue((byte)0x80,lidc));
		
		//kalau LID-M ada, tulis, kalau tidak, biarin aja
		if (lidm != null)
			dout.putDerValue(new DerValue((byte)0x81,lidm));
			
		//kalau XID ada, tulis, kalau tidak, biarin aja
		if (xid != null)
			dout.putDerValue(new DerValue((byte)0x82,xid));
		
		//tulis tag sequence dan panjangnya
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		//tulis semuanya ke dout
		out.write(dout2.toByteArray());	
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the LID-C
	 *
	 * @return Local ID; convenience label generated by and for Cardholder system
	 */
	public byte[] getLIDC() {
		return lidc;
	}

	/**
	 * Method for get the LID-M
	 *
	 * @return Local ID; convenience label generated by and for Merchant system 
	 */
	public byte[] getLIDM() {
		return lidm;
	}

	/**
	 * Method for get the LID-C
	 *
	 * @return globally unique ID generated by Merchant in PInitRes, or by Cardholder in PReq

	 */
	public byte[] getXID() {
		return xid;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageIDs object
	 */
	public String toString() {
		String s = "\n";
		if(lidc!=null){
			HexDumpEncoder hde = new HexDumpEncoder();
        		s = s + "MessageIDs ::= LID-C \n" + hde.encodeBuffer(lidc) + "\n";
		}
		if(lidm!=null){
			HexDumpEncoder hde = new HexDumpEncoder();
        		s = s + "MessageIDs ::= LID-M \n" + hde.encodeBuffer(lidm) + "\n";
		}
		if(xid!=null){
			HexDumpEncoder hde = new HexDumpEncoder();
        		s = s + "MessageIDs ::= XID \n" + hde.encodeBuffer(xid) + "\n";
		}

		return s;
	}

	static void main(String[] args) {
		try 
		{
			byte[] lidc = new byte[20];
			byte[] lidm = new byte[20];
			byte[] xid = new byte[20];

			Random myRandom = new Random(5485076);
			myRandom.nextBytes(lidc);
			myRandom.setSeed(8642178);
			myRandom.nextBytes(lidm);
			myRandom.setSeed(7870481);
			myRandom.nextBytes(xid);

			MessageIDs mids = new MessageIDs(null,lidm,xid);
			System.out.println(mids);

			DerOutputStream dout = new DerOutputStream();
			mids.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			MessageIDs mids2 = new MessageIDs(new DerInputStream(dout.toByteArray()));
			System.out.println(mids2);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

